import random

from jmetal.core.problem import BinaryProblem, FloatProblem
from jmetal.core.solution import BinarySolution, FloatSolution

__author__ = "Antonio J. Nebro"


class OneMax(BinaryProblem):
    """ Class representing problem Kursawe """

    def __init__(self, number_of_bits: int = 256):
        self.number_of_bits = number_of_bits
        self.number_of_objectives = 1
        self.number_of_variables = 1
        self.number_of_constraints = 0

    def evaluate(self, solution: BinarySolution) -> None:
        counter_of_ones = 0
        for bits in solution.variables[0]:
            if bits:
                counter_of_ones += 1

        solution.objectives[0] = -1.0 * counter_of_ones

    def create_solution(self) -> BinarySolution:
        new_solution = BinarySolution(number_of_variables=1, number_of_objectives=1)
        new_solution.variables[0] = \
            [True if random.randint(0, 1) == 0 else False for i in range(self.number_of_bits)]
        return new_solution

    def get_name(self):
        return "OneMax"


class Sphere(FloatProblem):
    def __init__(self, number_of_variables: int = 10):
        super(Sphere, self).__init__()
        self.number_of_objectives = 1
        self.number_of_variables = number_of_variables
        self.number_of_constraints = 0

        self.lower_bound = [-5.12 for i in range(number_of_variables)]
        self.upper_bound = [5.12 for i in range(number_of_variables)]

        FloatSolution.lower_bound = self.lower_bound
        FloatSolution.upper_bound = self.upper_bound

    def evaluate(self, solution: FloatSolution):
        total = 0.0
        for x in solution.variables:
            total += x * x

        solution.objectives[0] = total

    def get_name(self):
        return "Sphere"



import logging
from typing import List

from jmetal.algorithm.multiobjective.nsgaii import NSGAII
from jmetal.component.observer import WriteFrontToFileObserver
from jmetal.core.solution import FloatSolution
from jmetal.operator.crossover import SBX
from jmetal.operator.mutation import Polynomial
from jmetal.operator.selection import BinaryTournament
from jmetal.problem.multiobjective.unconstrained import Kursawe
from jmetal.util.solution_list_output import SolutionListOutput

from jmetal.util.time import get_time_of_execution

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


@get_time_of_execution
def main() -> None:
    problem = Kursawe()
    algorithm = NSGAII[FloatSolution, List[FloatSolution]](
        problem,
        population_size=100,
        max_evaluations=25000,
        mutation=Polynomial(1.0/problem.number_of_variables, distribution_index=20),
        crossover=SBX(1.0, distribution_index=20),
        selection=BinaryTournament())

    observer = WriteFrontToFileObserver("output_directory")
    algorithm.observable.register(observer=observer)

    algorithm.run()

    result = algorithm.get_result()

    SolutionListOutput[FloatSolution].print_function_values_to_file("FUN."+problem.get_name(), result)

    logger.info("Algorithm (continuous problem): " + algorithm.get_name())
    logger.info("Problem: " + problem.get_name())

if __name__ == '__main__':
    main()



import time
import logging

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


def get_time_of_execution(fn):
    """ Decorator to get time of execution of any method inside a class. """

    def wrapped(*args, **kwargs):
        start_time = time.time()
        res = fn(*args, **kwargs)
        logger.info("Computing time to " + fn.__name__ + " (in seconds): " + str(time.time() - start_time))

        return res

    return wrapped



import logging

from jmetal.algorithm.singleobjective.evolutionaryalgorithm import GenerationalGeneticAlgorithm
from jmetal.component.observer import BasicAlgorithmConsumer

from jmetal.core.solution import FloatSolution
from jmetal.operator.crossover import SBX
from jmetal.operator.mutation import Polynomial
from jmetal.operator.selection import BinaryTournament
from jmetal.problem.singleobjective.unconstrained import Sphere

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


def main() -> None:
    variables = 10
    problem = Sphere(variables)
    algorithm = GenerationalGeneticAlgorithm[FloatSolution, FloatSolution](
        problem,
        population_size=100,
        max_evaluations=25000,
        mutation=Polynomial(1.0 / variables, distribution_index=20),
        crossover=SBX(1.0, distribution_index=20),
        selection=BinaryTournament())

    observer = BasicAlgorithmConsumer(2000)

    algorithm.observable.register(observer=observer)

    algorithm.start()
    algorithm.join()

    result = algorithm.get_result()
    logger.info("Algorithm: " + algorithm.get_name())
    logger.info("Problem: " + problem.get_name())
    logger.info("Solution: " + str(result.variables))
    logger.info("Fitness:  " + str(result.objectives[0]))


if __name__ == '__main__':
    main()



import unittest

import pytest

from jmetal.component.density_estimator import CrowdingDistance
from jmetal.core.solution import Solution


class CrowdingDistanceTestCases(unittest.TestCase):

    def setUp(self):
        self.crowding = CrowdingDistance()

    @pytest.fixture
    def test_should_the_crowding_distance_of_an_empty_set_do_nothing(self):
        solution_list = []
        self.crowding.compute_density_estimator(solution_list)

    def test_should_the_crowding_distance_of_single_solution_be_infinity(self):
        solution = Solution(3, 3)
        solution_list = [solution]

        self.crowding.compute_density_estimator(solution_list)
        value = solution_list[0].attributes["distance"]

        self.assertEqual(float("inf"), value)

    def test_should_the_crowding_distance_of_two_solutions_be_infinity(self):
        solution1 = Solution(3, 3)
        solution2 = Solution(3, 3)
        solution_list = [solution1, solution2]

        self.crowding.compute_density_estimator(solution_list)
        value_from_solution1 = solution_list[0].attributes["distance"]
        value_from_solution2 = solution_list[1].attributes["distance"]

        self.assertEqual(float("inf"), value_from_solution1)
        self.assertEqual(float("inf"), value_from_solution2)

    def test_should_the_crowding_distance_of_three_solutions_correctly_assigned(self):
        solution1 = Solution(2, 2)
        solution2 = Solution(2, 2)
        solution3 = Solution(2, 2)

        solution1.objectives[0] = 0.0
        solution1.objectives[1] = 1.0
        solution2.objectives[0] = 1.0
        solution2.objectives[1] = 0.0
        solution3.objectives[0] = 0.5
        solution3.objectives[1] = 0.5

        solution_list = [solution1, solution2, solution3]

        self.crowding.compute_density_estimator(solution_list)

        value_from_solution1 = solution_list[0].attributes["distance"]
        value_from_solution2 = solution_list[1].attributes["distance"]
        value_from_solution3 = solution_list[2].attributes["distance"]

        self.assertEqual(float("inf"), value_from_solution1)
        self.assertEqual(float("inf"), value_from_solution2)
        self.assertEqual(2.0, value_from_solution3)

    def test_should_the_crowding_distance_of_four_solutions_correctly_assigned(self):
        solution1 = Solution(2, 2)
        solution2 = Solution(2, 2)
        solution3 = Solution(2, 2)
        solution4 = Solution(2, 2)

        solution1.objectives[0] = 0.0
        solution1.objectives[1] = 1.0
        solution2.objectives[0] = 1.0
        solution2.objectives[1] = 0.0
        solution3.objectives[0] = 0.5
        solution3.objectives[1] = 0.5
        solution4.objectives[0] = 0.75
        solution4.objectives[1] = 0.75

        solution_list = [solution1, solution2, solution3, solution4]

        self.crowding.compute_density_estimator(solution_list)

        value_from_solution1 = solution_list[0].attributes["distance"]
        value_from_solution2 = solution_list[1].attributes["distance"]
        value_from_solution3 = solution_list[2].attributes["distance"]
        value_from_solution4 = solution_list[3].attributes["distance"]

        self.assertEqual(float("inf"), value_from_solution1)
        self.assertEqual(float("inf"), value_from_solution2)
        self.assertGreater(value_from_solution3, value_from_solution4)


if __name__ == "__main__":
    unittest.main()


from typing import TypeVar, List

from jmetal.algorithm.singleobjective.evolutionaryalgorithm import GenerationalGeneticAlgorithm
from jmetal.core.operator import Mutation, Crossover, Selection
from jmetal.core.problem import Problem
from jmetal.operator.selection import RankingAndCrowdingDistanceSelection
from jmetal.util.observable import Observable, DefaultObservable

S = TypeVar('S')
R = TypeVar(List[S])


class NSGAII(GenerationalGeneticAlgorithm[S, R]):
    def __init__(self,
                 problem: Problem[S],
                 population_size: int,
                 max_evaluations: int,
                 mutation: Mutation[S],
                 crossover: Crossover[S, S],
                 selection: Selection[List[S], S],
                 observable: Observable = DefaultObservable()):
        super(NSGAII, self).__init__(
            problem,
            population_size,
            max_evaluations,
            mutation,
            crossover,
            selection,
            observable)

    def replacement(self, population: List[S], offspring_population: List[S]) -> List[List[TypeVar('S')]]:
        join_population = population + offspring_population
        return RankingAndCrowdingDistanceSelection(self.population_size).execute(join_population)

    def get_name(self) -> str:
        return "NSGA-II"

    def get_result(self) -> R:
        return self.population





from typing import List, Generic, TypeVar

__author__ = "Antonio J. Nebro"

BitSet = List[bool]
S = TypeVar('S')


class Solution(Generic[S]):
    """ Class representing solutions """

    def __init__(self, number_of_variables : int, number_of_objectives : int, number_of_constraints = 0):
        self.number_of_objectives: int = number_of_objectives
        self.number_of_variables:int = number_of_variables
        self.number_of_constraints: int = number_of_constraints
        self.objectives = [0.0 for x in range(self.number_of_objectives)]
        self.variables = [[] for x in range(self.number_of_variables)]
        self.attributes = {}


class BinarySolution(Solution[BitSet]):
    """ Class representing float solutions """

    def __init__(self, number_of_variables: int, number_of_objectives: int, number_of_constraints=0):
        super(BinarySolution, self).__init__(number_of_variables, number_of_objectives, number_of_constraints)

    def get_total_number_of_bits(self) -> int:
        total = 0
        for var in self.variables:
            total += len(var)

        return total


class FloatSolution(Solution[float]):
    """ Class representing float solutions """

    def __init__(self, number_of_variables : int, number_of_objectives : int, number_of_constraints : int,
                 lower_bound : List[float], upper_bound : List[float]):
        super(FloatSolution, self).__init__(number_of_variables, number_of_objectives, number_of_constraints)
        self.lower_bound = lower_bound
        self.upper_bound = upper_bound


class IntegerSolution(Solution[int]):
    """ Class representing integer solutions """

    def __init__(self, number_of_variables : int, number_of_objectives : int, number_of_constraints : int,
                 lower_bound : List[int], upper_bound : List[int]):
        super(IntegerSolution, self).__init__(number_of_variables, number_of_objectives, number_of_constraints)
        self.lower_bound = lower_bound
        self.upper_bound = upper_bound


from jmetal.algorithm.singleobjective.evolutionaryalgorithm import NonElitistEvolutionStrategy
from jmetal.core.solution import BinarySolution
from jmetal.operator.mutation import BitFlip
from jmetal.problem.singleobjective.unconstrained import OneMax


def main() -> None:
    bits = 512
    problem = OneMax(bits)
    algorithm = NonElitistEvolutionStrategy[BinarySolution, BinarySolution]\
        (problem, mu=1, lambdA=10, max_evaluations= 25000, mutation=BitFlip(1.0/bits))

    algorithm.run()
    result = algorithm.get_result()
    print("Algorithm: " + algorithm.get_name())
    print("Problem: " + problem.get_name())
    print("Solution: " + str(result.variables[0]))
    print("Fitness:  " + str(result.objectives[0]))
    print("Computing time: " + str(algorithm.total_computing_time))

if __name__ == '__main__':
    main()



import unittest

from jmetal.component.archive import NonDominatedSolutionListArchive
from jmetal.core.solution import Solution


class NonDominatedTestCases(unittest.TestCase):

    def setUp(self):
        pass

    def test_should_constructor_create_a_non_null_object(self):
        archive = NonDominatedSolutionListArchive[Solution]()
        self.assertIsNotNone(archive)

    def test_should_adding_one_solution_work_properly(self):
        archive = NonDominatedSolutionListArchive[Solution]()
        solution = Solution(1,1)
        archive.add(solution)
        self.assertEqual(1, archive.size())
        self.assertEqual(solution, archive.get_solution_list()[0])

    def test_should_adding_two_solutions_work_properly_if_one_is_dominated(self):
        dominated_solution = Solution(1,2)
        dominated_solution.objectives = [2.0,2.0]

        dominant_solution = Solution(1,2)
        dominant_solution.objectives = [1.0,1.0]

        archive = NonDominatedSolutionListArchive[Solution]()
        archive.add(dominated_solution)
        archive.add(dominant_solution)

        self.assertEqual(1, archive.size())
        self.assertEqual(dominant_solution, archive.get_solution_list()[0])

    def test_should_adding_two_solutions_work_properly_if_both_are_non_dominated(self):
        solution1 = Solution(1, 2)
        solution1.objectives = [1.0, 0.0]

        solution2 = Solution(1, 2)
        solution2.objectives = [0.0, 1.0]

        archive = NonDominatedSolutionListArchive[Solution]()
        archive.add(solution1)
        archive.add(solution2)

        self.assertEqual(2, archive.size())
        self.assertTrue(solution1 in archive.get_solution_list() and solution2 in archive.get_solution_list())


    def test_should_adding_four_solutions_work_properly_if_one_dominates_the_others(self):
        solution1 = Solution(1, 2)
        solution1.objectives = [1.0, 1.0]

        solution2 = Solution(1, 2)
        solution2.objectives = [0.0, 2.0]

        solution3 = Solution(1, 2)
        solution3.objectives = [0.5, 1.5]

        solution4 = Solution(1, 2)
        solution4.objectives = [0.0, 0.0]

        archive = NonDominatedSolutionListArchive[Solution]()
        archive.add(solution1)
        archive.add(solution2)
        archive.add(solution3)
        archive.add(solution4)

        self.assertEqual(1, archive.size())
        self.assertEqual(solution4, archive.get_solution_list()[0])


    def test_should_adding_three_solutions_work_properly_if_two_of_them_are_equal(self):
        solution1 = Solution(1, 2)
        solution1.objectives = [1.0, 1.0]

        solution2 = Solution(1, 2)
        solution2.objectives = [0.0, 2.0]

        solution3 = Solution(1, 2)
        solution3.objectives = [1.0, 1.0]

        archive = NonDominatedSolutionListArchive[Solution]()
        archive.add(solution1)
        archive.add(solution2)
        result = archive.add(solution3)

        self.assertEqual(2, archive.size())
        self.assertFalse(result)
        self.assertTrue(solution1 in archive.get_solution_list() or solution3 in archive.get_solution_list())


if __name__ == '__main__':
    unittest.main()



from jmetal import algorithm
from jmetal import core
from jmetal import operator
from jmetal import problem

__all__ = ['core', 'algorithm', 'operator', 'problem']
__version__ = '1.0'
__author__ = 'Antonio J. Nebro <ajnebro@uma.es>'



import copy
import random
from typing import List

from jmetal.core.operator import Crossover
from jmetal.core.solution import Solution, FloatSolution, BinarySolution

""" Class implementing the null crossover operator """


class Null(Crossover[Solution, Solution]):
    def __init__(self):
        super(Null, self).__init__(probability=0)

    def execute(self, parents: List[Solution]) -> List[Solution]:
        if len(parents) != 2:
            raise Exception("The number of parents is not two: " + str(len(parents)))

        return [copy.deepcopy(parents[0]), copy.deepcopy(parents[1])]

    def get_number_of_parents(self):
        return 2
    

class SBX(Crossover[FloatSolution, FloatSolution]):
    __EPS = 1.0e-14

    def __init__(self, probability: float, distribution_index: float = 20.0):
        super(SBX, self).__init__(probability=probability)
        self.distribution_index = distribution_index

    def execute(self, parents: List[FloatSolution]) -> List[FloatSolution]:
        if len(parents) != 2:
            raise Exception("The number of parents is not two: " + str(len(parents)))

        offspring = [copy.deepcopy(parents[0]), copy.deepcopy(parents[1])]
        rand = random.random()

        if rand <= self.probability:
            for i in range(parents[0].number_of_variables):
                value_x1, value_x2 = parents[0].variables[i], parents[1].variables[i]

                if random.random() <= 0.5:
                    if abs(value_x1 - value_x2) > self.__EPS:
                        if value_x1 < value_x2:
                            y1, y2 = value_x1, value_x2
                        else:
                            y1, y2 = value_x2, value_x1

                        lowerBound, upperBound = parents[0].lower_bound[i], parents[1].upper_bound[i]

                        beta = 1.0 + (2.0 * (y1 - lowerBound) / (y2 - y1))
                        alpha = 2.0 - pow(beta, -(self.distribution_index + 1.0))

                        rand = random.random()
                        if rand <= (1.0 / alpha):
                            betaq = pow(rand * alpha, (1.0 / (self.distribution_index + 1.0)))
                        else:
                            betaq = pow(1.0 / (2.0 - rand * alpha), 1.0 / (self.distribution_index + 1.0))

                        c1 = 0.5 * (y1 + y2 - betaq * (y2 - y1))
                        beta = 1.0 + (2.0 * (upperBound - y2) / (y2 - y1))
                        alpha = 2.0 - pow(beta, -(self.distribution_index + 1.0))

                        if rand <= (1.0 / alpha):
                            betaq = pow((rand * alpha), (1.0 / (self.distribution_index + 1.0)))
                        else:
                            betaq = pow(1.0 / (2.0 - rand * alpha), 1.0 / (self.distribution_index + 1.0))

                        c2 = 0.5 * (y1 + y2 + betaq * (y2 - y1))

                        if c1 < lowerBound:
                            c1 = lowerBound
                        if c2 < lowerBound:
                            c2 = lowerBound
                        if c1 > upperBound:
                            c1 = upperBound
                        if c2 > upperBound:
                            c2 = upperBound

                        if random.random() <= 0.5:
                            offspring[0].variables[i] = c2
                            offspring[1].variables[i] = c1
                        else:
                            offspring[0].variables[i] = c1
                            offspring[1].variables[i] = c2
                    else:
                        offspring[0].variables[i] = value_x1
                        offspring[1].variables[i] = value_x2
                else:
                    offspring[0].variables[i] = value_x1
                    offspring[1].variables[i] = value_x2
        return offspring

    def get_number_of_parents(self):
        return 2


class SinglePoint(Crossover[BinarySolution, BinarySolution]):
    def __init__(self, probability: float):
        super(SinglePoint, self).__init__(probability=probability)

    def execute(self, parents: List[BinarySolution]) -> List[BinarySolution]:
        if len(parents) != 2:
            raise Exception("The number of parents is not two: " + str(len(parents)))

        offspring = [copy.deepcopy(parents[0]), copy.deepcopy(parents[1])]
        rand = random.random()

        if rand <= self.probability:
            # 1. Get the total number of bits
            total_number_of_bits = parents[0].get_total_number_of_bits()

            # 2. Calculate the point to make the crossover
            crossover_point = random.randrange(0, total_number_of_bits)

            # 3. Compute the variable containing the crossover bit
            variable_to_cut = 0
            bits_count = len(parents[1].variables[variable_to_cut])
            while bits_count < (crossover_point + 1):
                variable_to_cut += 1
                bits_count += len(parents[1].variables[variable_to_cut])

            # 4. Compute the bit into the selected variable
            diff = bits_count - crossover_point
            crossover_point_in_variable = len(parents[1].variables[variable_to_cut]) - diff

            # 5. Apply the crossover to the variable
            bitset1 = parents[0].variables[variable_to_cut]
            bitset2 = parents[1].variables[variable_to_cut]

            for i in range(crossover_point_in_variable, len(bitset1)):
                swap = bitset1[i]
                bitset1[i] = bitset2[i]
                bitset2[i] = swap

            offspring[0].variables[variable_to_cut] = bitset1
            offspring[1].variables[variable_to_cut] = bitset2

            # 6. Apply the crossover to the other variables
            for i in range(variable_to_cut + 1, parents[0].number_of_variables):
                offspring[0].variables[i] = copy.deepcopy(parents[1].variables[i])
                offspring[1].variables[i] = copy.deepcopy(parents[0].variables[i])

        return offspring

    def get_number_of_parents(self):
        return 2



import logging
import os

from jmetal.util.observable import Observer
from jmetal.util.solution_list_output import SolutionListOutput

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


class BasicAlgorithmConsumer(Observer):
    def __init__(self, frequency: float = 1.0) -> None:
        self.display_frequency = frequency

    def update(self, *args, **kwargs):
        evaluations = kwargs["evaluations"]
        if (evaluations % self.display_frequency) == 0:
            logger.info("Evaluations: " + str(evaluations) +
              ". Best fitness: " + str(kwargs["population"][0].objectives[0]) +
              ". Computing time: " + str(kwargs["computing time"]))


class WriteFrontToFileObserver(Observer):
    def __init__(self, output_directory) -> None:
        self.counter = 0
        self.directory = output_directory

        if os.path.isdir(self.directory):
            logger.info("Directory " + self.directory + " exists. Removing contents.")
            for file in os.listdir(self.directory):
                os.remove(self.directory + "/" + file)
        else:
            logger.info("Directory " + self.directory + " does not exist. Creating it.")
            os.mkdir(self.directory)

    def update(self, *args, **kwargs):
        SolutionListOutput.print_function_values_to_file(
            self.directory + "/FUN." + str(self.counter), kwargs["population"])

        self.counter += 1


class AlgorithmObserver(Observer):
    def __init__(self, animation_speed: float) -> None:
        self.animation_speed = animation_speed

    def update(self, *args, **kwargs):
        SolutionListOutput.plot_scatter_real_time(kwargs["population"], self.animation_speed)



from typing import TypeVar, List, Generic

from jmetal.component.archive import NonDominatedSolutionListArchive
from jmetal.core.problem import Problem

""" Class representing simple random search algorithms """


S = TypeVar('S')


class RandomSearch(Generic[S]):
    def __init__(self, problem: Problem[S], max_evaluations: int):
        self.problem = problem
        self.max_evaluations = max_evaluations
        self.archive = NonDominatedSolutionListArchive()

    def run(self) -> None:
        for i in range(self.max_evaluations):
            new_solution = self.problem.create_solution()
            self.problem.evaluate(new_solution)
            self.archive.add(new_solution)

    def get_name(self) -> str:
        return "Random Search Algorithm"

    def get_result(self) -> List[S]:
        return self.archive.get_solution_list()


#from jmetal.util.test.test_ranking import DominanceRankingTestCases


def pytest_addoption(parser):
    parser.addoption('--self', default=1, help="run many tests")


def pytest_generate_tests(metafunc):
    self = metafunc.config.getoption('--self')
    if 'self' in metafunc.fixturenames:
        for x in range(1, self):
            metafunc.parametrize("self", self)


"""
    ============================================================
    :mod:`zdt` -- ZDT problem family of multi-objective problems
    ============================================================

    This module provides the implementation of the ZDT problems defined in:

    Zitzler, E., Deb, K., Thiele, L.
    Comparison of multiobjective evolutionary algorithms: Empirical results.
    Evolutionary Computation 8(2), pp: 173-195. 2000

    DOI: https://doi.org/10.1162/106365600568202


    These problems are scalable in the number of the decision variables, i.e. the Pareto front
    is the same independently of the number of variables

    .. module:: jmetal.problem.multiobjective.zdt

    :platform: Unix, Windows
    :synopsis: Module including the implementation of the ZDT problem family.

    .. moduleauthor:: Antonio J. Nebro <antonio@lcc.uma.es>

"""


from math import sqrt, exp, pow, sin, pi, cos

from jmetal.core.problem import FloatProblem
from jmetal.core.solution import FloatSolution


class ZDT1(FloatProblem):
    """ Problem ZDT1

    .. note:: Bi-objective unconstrained problem. The default number of variables is 30.

    .. note:: Continuous problem having a convex Pareto front

    """
    def __init__(self, number_of_variables: int = 30):
        """Constructor

        Arguments:

        :param number_of_variables: number of decision variables of the problem
        """
        self.number_of_variables = number_of_variables
        self.number_of_objectives = 2
        self.number_of_constraints = 0

        self.lower_bound = self.number_of_variables * [0.0]
        self.upper_bound = self.number_of_variables * [1.0]

        FloatSolution.lower_bound = self.lower_bound
        FloatSolution.upper_bound = self.upper_bound

    def evaluate(self, solution: FloatSolution) -> None:
        """Evaluate method

        :param solution: Solution to be evaluated
        """
        g = self.__eval_g(solution)
        h = self.__eval_h(solution.variables[0], g)

        solution.objectives[0] = solution.variables[0]
        solution.objectives[1] = h * g

    def __eval_g(self, solution: FloatSolution):
        g = sum(solution.variables) - solution.variables[0]

        constant = 9.0 / (solution.number_of_variables - 1)
        g = constant * g
        g = g + 1.0
        return g

    def __eval_h(self, f: float, g: float) -> float:
        return 1.0 - sqrt(f/g)

    def get_name(self):
        return "ZDT1"


class ZDT2(FloatProblem):
    """ Problem ZDT2

    .. note:: Bi-objective unconstrained problem. The default number of variables is 30.

    .. note:: Continuous problem having a non-convex Pareto front

    """
    def __init__(self, number_of_variables: int = 30):
        self.number_of_variables = number_of_variables
        self.number_of_objectives = 2
        self.number_of_constraints = 0

        self.lower_bound = self.number_of_variables * [0.0]
        self.upper_bound = self.number_of_variables * [1.0]

        FloatSolution.lower_bound = self.lower_bound
        FloatSolution.upper_bound = self.upper_bound

    def evaluate(self, solution: FloatSolution) -> None:
        """Evaluate method

        :param solution: Solution to be evaluated
        """
        g = self.__eval_g(solution)
        h = self.__eval_h(solution.variables[0], g)

        solution.objectives[0] = solution.variables[0]
        solution.objectives[1] = h * g

    def __eval_g(self, solution: FloatSolution):
        g = sum(solution.variables) - solution.variables[0]

        constant = 9.0 / (solution.number_of_variables - 1)
        g = constant * g
        g = g + 1.0
        return g

    def __eval_h(self, f: float, g: float) -> float:
        return 1.0 - pow(f/g, 2.0)

    def get_name(self):
        return "ZDT2"


class ZDT3(FloatProblem):
    """ Problem ZDT3

    .. note:: Bi-objective unconstrained problem. The default number of variables is 30.

    .. note:: Continuous problem having a partitioned Pareto front

    """
    def __init__(self, number_of_variables: int = 30):
        self.number_of_variables = number_of_variables
        self.number_of_objectives = 2
        self.number_of_constraints = 0

        self.lower_bound = self.number_of_variables * [0.0]
        self.upper_bound = self.number_of_variables * [1.0]

        FloatSolution.lower_bound = self.lower_bound
        FloatSolution.upper_bound = self.upper_bound

    def evaluate(self, solution: FloatSolution) -> None:
        """Evaluate method

        :param solution: Solution to be evaluated
        """
        g = self.__eval_g(solution)
        h = self.__eval_h(solution.variables[0], g)

        solution.objectives[0] = solution.variables[0]
        solution.objectives[1] = h * g

    def __eval_g(self, solution: FloatSolution):
        g = sum(solution.variables) - solution.variables[0]

        constant = 9.0 / (solution.number_of_variables - 1)
        g = constant * g
        g = g + 1.0
        return g

    def __eval_h(self, f: float, g: float) -> float:
        return 1.0 - sqrt(f/g) - (f/g)*sin(10.0*f*pi)

    def get_name(self):
        return "ZDT3"


class ZDT4(FloatProblem):
    """ Problem ZDT4

    .. note:: Bi-objective unconstrained problem. The default number of variables is 10.

    .. note:: Continuous multi-modal problem having a convex Pareto front

    """
    def __init__(self, number_of_variables: int = 10):
        self.number_of_variables = number_of_variables
        self.number_of_objectives = 2
        self.number_of_constraints = 0

        self.lower_bound = self.number_of_variables * [-5.0]
        self.upper_bound = self.number_of_variables * [5.0]
        self.lower_bound[0] = 0.0
        self.upper_bound[0] = 1.0

        FloatSolution.lower_bound = self.lower_bound
        FloatSolution.upper_bound = self.upper_bound

    def evaluate(self, solution: FloatSolution) -> None:
        """Evaluate method

        :param solution: Solution to be evaluated
        """
        g = self.__eval_g(solution)
        h = self.__eval_h(solution.variables[0], g)

        solution.objectives[0] = solution.variables[0]
        solution.objectives[1] = h * g

    def __eval_g(self, solution: FloatSolution):
        g = 0.0

        for i in range(1, self.number_of_variables):
            g += pow(solution.variables[i], 2.0) - 10.0 * cos(4.0*pi*solution.variables[i])

        g += 1.0 + 10.0 * (solution.number_of_variables - 1)

        return g

    def __eval_h(self, f: float, g: float) -> float:
        return 1.0 - sqrt(f/g)

    def get_name(self):
        return "ZDT4"


class ZDT6(FloatProblem):
    """ Problem ZDT6

    .. note:: Bi-objective unconstrained problem. The default number of variables is 10.

    .. note:: Continuous problem having a non-convex Pareto front

    """
    def __init__(self, number_of_variables: int = 10):
        self.number_of_variables = number_of_variables
        self.number_of_objectives = 2
        self.number_of_constraints = 0

        self.lower_bound = self.number_of_variables * [0.0]
        self.upper_bound = self.number_of_variables * [1.0]

        FloatSolution.lower_bound = self.lower_bound
        FloatSolution.upper_bound = self.upper_bound

    def evaluate(self, solution: FloatSolution) -> None:
        """Evaluate method

        :param solution: Solution to be evaluated
        """
        f0 = 1.0 - exp((-4.0) * solution.variables[0]) \
                   * pow(sin(6.0 * pi * solution.variables[0]), 6.0)
        g = self.__eval_g(solution)
        h = self.__eval_h(f0, g)

        solution.objectives[0] = f0
        solution.objectives[1] = h * g

    def __eval_g(self, solution: FloatSolution):
        g = sum(solution.variables) - solution.variables[0]
        g = g / (solution.number_of_variables - 1)
        g = pow(g, 0.25)
        g = 9.0 * g
        g = 1.0 + g

        return g

    def __eval_h(self, f: float, g: float) -> float:
        return 1.0 - pow(f/g, 2.0)

    def get_name(self):
        return "ZDT6"



import logging

from jmetal.algorithm.singleobjective.evolutionaryalgorithm import GenerationalGeneticAlgorithm
from jmetal.core.solution import BinarySolution
from jmetal.operator.crossover import SinglePoint
from jmetal.operator.mutation import BitFlip
from jmetal.operator.selection import BinaryTournament
from jmetal.problem.singleobjective.unconstrained import OneMax

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


def main() -> None:
    bits = 512
    problem = OneMax(bits)
    algorithm = GenerationalGeneticAlgorithm[BinarySolution, BinarySolution](
        problem,
        population_size = 100,
        max_evaluations = 25000,
        mutation = BitFlip(1.0/bits),
        crossover = SinglePoint(0.9),
        selection = BinaryTournament())

    algorithm.run()
    result = algorithm.get_result()

    logger.info("Algorithm (binary problem): " + algorithm.get_name())
    logger.info("Problem: " + problem.get_name())
    logger.info("Solution: " + str(result.variables[0]))
    logger.info("Fitness:  " + str(result.objectives[0]))

if __name__ == '__main__':
    main()



import random
from typing import Generic, TypeVar

from jmetal.core.solution import BinarySolution, FloatSolution, IntegerSolution

__author__ = "Antonio J. Nebro"

S = TypeVar('S')


class Problem(Generic[S]):
    """ Class representing problems """

    def __init__(self):
        self.number_of_variables: int = 0
        self.number_of_objectives: int = 0
        self.number_of_constraints: int = 0

    def evaluate(self, solution: S) -> None:
        pass

    def create_solution(self) -> S:
        pass


class BinaryProblem(BinarySolution):
    """ Class representing binary problems """

    def evaluate(self, solution: BinarySolution) -> None:
        pass

    def create_solution(self) -> BinarySolution:
        pass


class FloatProblem(FloatSolution):
    """ Class representing float problems """
    def __init__(self):
        self.lower_bound = []
        self.upper_bound = []

    def evaluate(self, solution: FloatSolution) -> None:
        pass

    def create_solution(self) -> FloatSolution:
        new_solution = FloatSolution(self.number_of_variables, self.number_of_objectives, self.number_of_constraints,
                                     self.lower_bound, self.upper_bound)
        new_solution.variables = \
            [random.uniform(self.lower_bound[i]*1.0, self.upper_bound[i]*1.0) for i in range(self.number_of_variables)]

        return new_solution


class IntegerProblem(IntegerSolution):
    """ Class representing integer problems """

    def __init__(self):
        self.lower_bound = []
        self.upper_bound = []

    def evaluate(self, solution: IntegerSolution) -> None:
        pass

    def create_solution(self) -> IntegerSolution:
        new_solution = IntegerSolution(
            self.number_of_variables,
            self.number_of_objectives,
            self.number_of_constraints,
            self.lower_bound, self.upper_bound)

        new_solution.variables = \
            [int(random.uniform(self.lower_bound[i]*1.0, self.upper_bound[i]*1.0)) for i in range(self.number_of_variables)]

        return new_solution



import logging
import threading
import time
from typing import TypeVar, Generic, List

from jmetal.util.time import get_time_of_execution

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

S = TypeVar('S')
R = TypeVar('R')


class Algorithm(Generic[S, R], threading.Thread):
    def __init__(self):
        threading.Thread.__init__(self)
        self.observable = None
        self.evaluations: int = 0
        self.start_computing_time: int = time.process_time()

    def get_name(self) -> str:
        pass

    def get_evaluations(self) -> int:
        return self.evaluations

    def get_current_computing_time(self) -> float:
        return time.process_time() - self.start_computing_time


class EvolutionaryAlgorithm(Algorithm[S, R]):
    def __init__(self):
        super(EvolutionaryAlgorithm,self).__init__()
        self.population = []

    def create_initial_population(self) -> List[S]:
        pass

    def evaluate_population(self, population: List[S]) -> List[S]:
        pass

    def init_progress(self) -> None:
        pass

    def is_stopping_condition_reached(self) -> bool:
        pass

    def selection(self, population: List[S]) -> List[S]:
        pass

    def reproduction(self, population: List[S]) -> List[S]:
        pass

    def replacement(self, population: List[S], offspring_population: List[S]) -> List[S]:
        pass

    def update_progress(self):
        pass

    def get_result(self) -> R:
        pass

    @get_time_of_execution
    def run(self):
        """
        Step One: Generate the initial population of individuals randomly. (First generation)
        Step Two: Evaluate the fitness of each individual in that population (time limit, sufficient fitness achieved, etc.)
        Step Three: Repeat the following regenerational steps until termination:
            1. Select the best-fit individuals for reproduction. (Parents)
            2. Breed new individuals through crossover and mutation operations to give birth to offspring.
            3. Evaluate the individual fitness of new individuals.
            4. Replace least-fit population with new individuals.
        """

        self.population = self.create_initial_population() # Step One
        self.population = self.evaluate_population(self.population) # Step Two
        self.init_progress()

        while not self.is_stopping_condition_reached(): # Step Three
            mating_population = self.selection(self.population) # Step Three.1
            offspring_population = self.reproduction(mating_population) # Step Three.2
            offspring_population = self.evaluate_population(offspring_population) # Step Three.3
            self.population = self.replacement(self.population, offspring_population) # Step Three.4
            self.update_progress()



from typing import TypeVar, List

S = TypeVar('S')

"""
.. module:: crowding_distance
   :platform: Unix, Windows
   :synopsis: Crowding distance implementation.

.. moduleauthor:: Álvaro Gómez Jáuregui <alvarogj@lcc.uma.es>
"""


class DensityEstimator(List[S]):
    """This is the interface of any density estimator algorithm.
    """

    def compute_density_estimator(self, solution_list: List[S]) -> float:
        pass


class CrowdingDistance(DensityEstimator[List[S]]):
    """This class implements a DensityEstimator based on the crowding distance.
    In consequence, the main method of this class is :func:`compute_density_estimator`.
    """

    def compute_density_estimator(self, solution_list: List[S]):
        """This function performs the computation of the crowding density estimation over the solution list.
            .. note::
               This method assign the distance in the inner elements of the solution list.

        :param solution_list: The list of solutions.
        """
        size = len(solution_list)

        if size is 0:
            return
        elif size is 1:
            solution_list[0].attributes["distance"] = float("inf")
            return
        elif size is 2:
            solution_list[0].attributes["distance"] = float("inf")
            solution_list[1].attributes["distance"] = float("inf")
            return

        for i in range(len(solution_list)):
            solution_list[i].attributes["distance"] = 0.0

        number_of_objectives = solution_list[0].number_of_objectives

        for i in range(number_of_objectives):
            # Sort the population by Obj n
            solution_list = sorted(solution_list, key=lambda x: x.objectives[i])
            objective_minn = solution_list[0].objectives[i]
            objective_maxn = solution_list[len(solution_list) - 1].objectives[i]

            # Set de crowding distance
            solution_list[0].attributes["distance"] = float("inf")
            solution_list[size - 1].attributes["distance"] = float("inf")

            for j in range(1, size - 1):
                distance = solution_list[j + 1].objectives[i] - solution_list[j - 1].objectives[i]
                distance = distance / (objective_maxn - objective_minn)
                distance += solution_list[j].attributes["distance"]
                solution_list[j].attributes["distance"] = distance



import logging
<<<<<<< HEAD
=======
import unittest
>>>>>>> ddedb114fb49743e93b66ae38ba9d4ec05c34168

from jmetal.algorithm.singleobjective.evolutionaryalgorithm import GenerationalGeneticAlgorithm
from jmetal.core.solution import BinarySolution
from jmetal.operator.crossover import SinglePoint
from jmetal.operator.mutation import BitFlip
from jmetal.operator.selection import BinaryTournament
from jmetal.problem.singleobjective.unconstrained import OneMax

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


<<<<<<< HEAD
import unittest


=======
>>>>>>> ddedb114fb49743e93b66ae38ba9d4ec05c34168
class AlgorithmTestIntegrationTestCases(unittest.TestCase):

    def setUp(self):
        bits = 512
        self.problem = OneMax(bits)
        self.algorithm = GenerationalGeneticAlgorithm[BinarySolution, BinarySolution](
            self.problem,
            population_size=100,
            max_evaluations=25000,
            mutation=BitFlip(1.0 / bits),
            crossover=SinglePoint(0.9),
            selection=BinaryTournament())


    def test_genetic_algorithm(self):
        self.algorithm.run()
        result = self.algorithm.get_result()
        logger.info("Algorithm (binary problem): " + self.algorithm.get_name())
        logger.info("Problem: " +  self.problem.get_name())
        logger.info("Solution: " + str(result.variables[0]))
        logger.info("Fitness:  " + str(result.objectives[0]))

        assert(-500 <= result.objectives[0] <= -300 or 300 <= result.objectives[0] <= 500)





import logging
from typing import TypeVar, List, Generic, Tuple

from jmetal.util.graphic import ScatterPlot
import matplotlib.pyplot as plt

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

S = TypeVar('S')


class SolutionListOutput(Generic[S]):
    @staticmethod
    def plot_scatter_to_file(file_name, solution_list:List[S]):
        """ Plot non-dominated solutions. For problems with TWO variables.
        """
        sc = ScatterPlot(plot_title="Pareto frontier")
        sc.simple_plot(solution_list, file_name=file_name)

    @staticmethod
    def plot_scatter_to_screen(solution_list:List[S]):
        """ Plot non-dominated solutions. For problems with TWO variables.
        """
        sc = ScatterPlot(plot_title="Pareto frontier (interactive)")
        sc.interactive_plot(solution_list)

    @staticmethod
    def plot_scatter_real_time(solution_list: List[S], animation_speed: float):
        """ Plot non-dominated solutions in real-time. For problems with TWO variables.
        """
        global sc

        if not plt.get_fignums():
            # The first time, set up plot
            sc = ScatterPlot(plot_title="Pareto frontier (real-time)", animation_speed=animation_speed)
            sc.simple_plot(solution_list, save=False)
        else:
            sc.update(solution_list)

    @staticmethod
    def print_variables_to_screen(solution_list: List[S]):
        for solution in solution_list:
            print(solution.variables[0])

    @staticmethod
    def print_function_values_to_screen(solution_list: List[S]):
        for solution in solution_list:
            print(str(solution_list.index(solution)) + ": ", sep='  ', end='', flush=True)
            print(solution.objectives, sep='  ', end='', flush=True)
            print()

    @staticmethod
    def print_function_values_to_file(file_name, solution_list: List[S]):
        logger.info("Output file (function values): " + file_name)
        with open(file_name, 'w') as of:
            for solution in solution_list:
                for function_value in solution.objectives:
                    of.write(str(function_value) + " ")
                of.write("\n")



import logging
from typing import List, TypeVar

from jmetal.algorithm.multiobjective.nsgaii import NSGAII
from jmetal.core.solution import FloatSolution
from jmetal.operator.crossover import SBX
from jmetal.operator.mutation import Polynomial
from jmetal.operator.selection import BinaryTournament
from jmetal.problem.multiobjective.unconstrained import Fonseca
from jmetal.util.solution_list_output import SolutionListOutput

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

S = TypeVar('S')
R = TypeVar(List[S])


def main() -> None:
    class NSGA2b(NSGAII[S, R]):
        def is_stopping_condition_reached(self):
            # Re-define the stopping condition
            reached = [False, True][self.get_current_computing_time() > 4]

            if reached:
                logger.info("Stopping condition reached!")

            return reached

    problem = Fonseca()
    algorithm = NSGA2b[FloatSolution, List[FloatSolution]](
        problem,
        population_size=100,
        max_evaluations=25000,
        mutation=Polynomial(1.0/problem.number_of_variables, distribution_index=20),
        crossover=SBX(1.0, distribution_index=20),
        selection=BinaryTournament())

    algorithm.run()
    result = algorithm.get_result()

    SolutionListOutput[FloatSolution].print_function_values_to_file("FUN."+problem.get_name(), result)

    logger.info("Algorithm (continuous problem): " + algorithm.get_name())
    logger.info("Problem: " + problem.get_name())

if __name__ == '__main__':
    main()



from typing import TypeVar, Generic, List

from jmetal.util.comparator import dominance_comparator, equal_solutions_comparator

S = TypeVar('S')


class Archive(Generic[S]):
    def add(self, solution: S) -> bool:
        pass

    def get(self, index:int) -> S:
        pass

    def get_solution_list(self) -> List[S]:
        pass

    def size(self) -> int:
        pass


class NonDominatedSolutionListArchive(Archive[S]):
    def __init__(self):
        self.solution_list = []

    def add(self, solution:S) -> bool:
        is_dominated = False
        is_contained = False

        if len(self.solution_list) == 0:
            self.solution_list.append(solution)
            return True
        else:
            number_of_deleted_solutions = 0

            # New copy of list and enumerate
            for index, current_solution in enumerate(list(self.solution_list)):
                is_dominated_flag = dominance_comparator(solution, current_solution)
                if is_dominated_flag == -1:
                    del self.solution_list[index-number_of_deleted_solutions]
                    number_of_deleted_solutions += 1
                elif is_dominated_flag == 1:
                    is_dominated = True
                    break
                elif is_dominated_flag == 0:
                    if equal_solutions_comparator(solution, current_solution) == 0:
                        is_contained = True
                        break

        if not is_dominated and not is_contained:
            self.solution_list.append(solution)
            return True

        return False

    def get(self, index: int) -> S:
        return self.solution_list[index]

    def get_solution_list(self) -> List[S]:
        return self.solution_list

    def size(self) -> int:
        return len(self.solution_list)


import unittest

from jmetal.util.observable import DefaultObservable, Observer


class ObservableTestCases(unittest.TestCase):
    def setUp(self):
        self.observable = DefaultObservable()

    def test_should_register_add_one_observer(self):
        observer = Observer()
        self.observable.register(observer)

        self.assertEqual(1, len(self.observable.observers))

    def test_should_register_add_two_observers(self):
        self.observable.register(Observer())
        self.observable.register(Observer())

        self.assertEqual(2, len(self.observable.observers))

    def test_should_register_add_two_observer(self):
        self.observable.register(Observer())
        self.observable.register(Observer())

        self.assertEqual(2, len(self.observable.observers))

    def test_should_deregister_remove_the_observer_if_it_is_registered(self):
        observer = Observer()
        self.observable.register(observer)
        self.observable.register(Observer())
        self.observable.deregister(observer)

        self.assertEqual(1, len(self.observable.observers))

    def test_should_deregister_not_remove_the_observer_if_it_is_not_registered(self):
        observer = Observer()
        observer2 = Observer()
        self.observable.register(observer)
        self.observable.deregister(observer2)

        self.assertEqual(1, len(self.observable.observers))
        self.assertTrue(observer in self.observable.observers)
        self.assertFalse(observer2 in self.observable.observers)

    def test_should_deregister_all_remove_all_the_observers(self):
        self.observable.register(Observer())
        self.observable.register(Observer())
        self.observable.register(Observer())
        self.observable.register(Observer())
        self.observable.deregister_all()

        self.assertEqual(0, len(self.observable.observers))

"""
    def test_should_notify_all_update_an_observer(self):
        observer = mock(Observer)
        self.observable.register(observer)

        self.observable.notify_all("color", "blue")
"""

if __name__ == "__main__":
    unittest.main()


import logging

from jmetal.algorithm.singleobjective.evolutionaryalgorithm import GenerationalGeneticAlgorithm
from jmetal.core.solution import FloatSolution
from jmetal.operator.crossover import SBX
from jmetal.operator.mutation import Polynomial
from jmetal.operator.selection import BinaryTournament
from jmetal.problem.singleobjective.unconstrained import Sphere

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


def main() -> None:
    variables = 10
    problem = Sphere(variables)
    algorithm = GenerationalGeneticAlgorithm[FloatSolution, FloatSolution](
        problem,
        population_size = 100,
        max_evaluations = 25000,
        mutation = Polynomial(1.0/variables, distribution_index=20),
        crossover = SBX(1.0, distribution_index=20),
        selection = BinaryTournament())

    algorithm.start()
    logger.info("Algorithm (running as a thread): " + algorithm.get_name())
    logger.info("Problem: " + problem.get_name())

    algorithm.join()
    result = algorithm.get_result()

    logger.info("Solution: " + str(result.variables))
    logger.info("Fitness:  " + str(result.objectives[0]))
    logger.info("Computing time: " + str(algorithm.total_computing_time))


if __name__ == '__main__':
    main()



from typing import TypeVar, Generic, List

__author__ = "Antonio J. Nebro"

Source = TypeVar('S')
Result = TypeVar('R')


class Operator(Generic[Source, Result]):
    """ Class representing operators """

    def execute(self, source: Source) -> Result:
        pass

    def get_name(self):
        pass


class Mutation(Operator[Source, Source]):
    """ Class representing mutation operators """

    def __init__(self, probability: float):
        if probability > 1.0:
            raise Exception("The probability is greater than one: " + str(probability))
        elif probability < 0.0:
            raise Exception("The probability is lower than zero: " + str(probability))

        self.probability = probability

    def execute(self, source: Source) -> Source:
        pass


class Crossover(Operator[List[Source], List[Result]]):
    def __init__(self, probability: float):
        if probability > 1.0:
            raise Exception("The probability is greater than one: " + str(probability))
        elif probability < 0.0:
            raise Exception("The probability is lower than zero: " + str(probability))

        self.probability = probability

    def execute(self, source: Source) -> Result:
        pass

    def get_number_of_parents(self) -> int:
        pass


class Selection(Operator[Source, Result]):
    def __init__(self):
        super(Selection, self).__init__()

    def execute(self, source: Source) -> Result:
        pass



from math import sqrt, exp, pow, sin

from jmetal.core.problem import FloatProblem
from jmetal.core.solution import FloatSolution

""" Test functions for multi-objective optimization """


class Kursawe(FloatProblem):
    """ Class representing problem Kursawe """

    def __init__(self, number_of_variables: int = 3):
        self.number_of_objectives = 2
        self.number_of_variables = number_of_variables
        self.number_of_constraints = 0

        self.lower_bound = [-5.0 for i in range(number_of_variables)]
        self.upper_bound = [5.0 for i in range(number_of_variables)]

        FloatSolution.lower_bound = self.lower_bound
        FloatSolution.upper_bound = self.upper_bound

    def evaluate(self, solution: FloatSolution) -> None:
        fx = [0.0 for x in range(self.number_of_objectives)]

        fx[0] = 0.0
        for i in range(self.number_of_variables - 1):
            xi = solution.variables[i] * solution.variables[i]
            xj = solution.variables[i + 1] * solution.variables[i + 1]
            aux = -0.2 * sqrt(xi + xj)
            fx[0] += -10 * exp(aux)

        fx[1] = 0.0
        for i in range(self.number_of_variables):
            fx[1] += pow(abs(solution.variables[i]), 0.8) + 5.0 * sin(pow(solution.variables[i], 3.0))

        solution.objectives[0] = fx[0]
        solution.objectives[1] = fx[1]

    def get_name(self):
        return "Kursawe"


class Fonseca(FloatProblem):
    def __init__(self):
        self.number_of_variables = 3
        self.number_of_objectives = 2
        self.number_of_constraints = 0

        self.lower_bound = self.number_of_variables * [-4]
        self.upper_bound = self.number_of_variables * [ 4]

        FloatSolution.lower_bound = self.lower_bound
        FloatSolution.upper_bound = self.upper_bound

    def evaluate(self, solution: FloatSolution):
        n = self.number_of_variables
        solution.objectives[0] = 1 - exp(-sum([(x - 1.0 / n ** 0.5) ** 2 for x in solution.variables]))
        solution.objectives[1] = 1 - exp(-sum([(x + 1.0 / n ** 0.5) ** 2 for x in solution.variables]))

    def get_name(self):
        return "Fonseca"


class Schaffer(FloatProblem):
    def __init__(self):
        self.number_of_variables = 1
        self.number_of_objectives = 2
        self.number_of_constraints = 0

        self.lower_bound = [-100000]
        self.upper_bound = [100000]

        FloatSolution.lower_bound = self.lower_bound
        FloatSolution.upper_bound = self.upper_bound

    def evaluate(self, solution: FloatSolution):
        value = solution.variables[0]

        solution.objectives[0] = value**2
        solution.objectives[1] = (value-2)**2

    def get_name(self):
        return "Schaffer"


class Viennet2(FloatProblem):
    def __init__(self):
        self.number_of_variables = 2
        self.number_of_objectives = 3
        self.number_of_constraints = 0

        self.lower_bound = self.number_of_variables * [-4]
        self.upper_bound = self.number_of_variables * [4]

        FloatSolution.lower_bound = self.lower_bound
        FloatSolution.upper_bound = self.upper_bound

    def evaluate(self, solution: FloatSolution):
        x0 = solution.variables[0]
        x1 = solution.variables[1]

        f1 = (x0 - 2) * (x0 - 2) / 2.0 + (x1 + 1) * (x1 + 1) / 13.0 + 3.0
        f2 = (x0 + x1 - 3.0) * (x0 + x1 - 3.0) / 36.0 + (-x0 + x1 + 2.0) * (-x0 + x1 + 2.0) / 8.0 - 17.0
        f3 = (x0 + 2 * x1 - 1) * (x0 + 2 * x1 - 1) / 175.0 + (2 * x1 - x0) * (2 * x1 - x0) / 17.0 - 13.0

        solution.objectives[0] = f1
        solution.objectives[1] = f2
        solution.objectives[2] = f3

    def get_name(self):
        return "Viennet2"



import random
from typing import List, TypeVar

from jmetal.component.density_estimator import CrowdingDistance
from jmetal.core.operator import Selection
from jmetal.util.comparator import dominance_comparator
from jmetal.util.ranking import DominanceRanking

""" Class implementing a best solution selection operator """

S = TypeVar('S')


class BinaryTournament(Selection[List[S], S]):
    def __init__(self):
        super(BinaryTournament, self).__init__()

    def get_name(self):
        return "Bynary tournament selection"

    def execute(self, solution_list: List[S]) -> S:
        if solution_list is None:
            raise Exception("The solution list is null")
        elif len(solution_list) == 0:
            raise Exception("The solution is empty")

        if len(solution_list) == 1:
            result = solution_list[0]
        else:
            i, j = random.sample(range(0, len(solution_list)), 2)  # sampling without replacement
            solution1 = solution_list[i]
            solution2 = solution_list[j]

            flag = dominance_comparator(solution1, solution2)

            if flag == -1:
                result = solution1
            elif flag == 1:
                result = solution2
            else:
                result = [solution1, solution2][random.random() < 0.5]

        return result


class BestSolution(Selection[List[S], S]):
    def __init__(self):
        super(BestSolution, self).__init__()

    def execute(self, solution_list: List[S]) -> S:
        if solution_list is None:
            raise Exception("The solution list is null")
        elif len(solution_list) == 0:
            raise Exception("The solution is empty")

        result = solution_list[0]
        for solution in solution_list[1:]:
            if dominance_comparator(solution, result) < 0:
                result = solution

        return result


class NaryRandomSolution(Selection[List[S], S]):
    def __init__(self, number_of_solutions_to_be_returned:int = 1):
        super(NaryRandomSolution, self).__init__()
        if number_of_solutions_to_be_returned < 0:
            raise Exception("The number of solutions to be returned must be positive integer")

        self.number_of_solutions_to_be_returned = number_of_solutions_to_be_returned

    def execute(self, solution_list: List[S]) -> S:
        if solution_list is None:
            raise Exception("The solution list is null")
        if len(solution_list) == 0:
            raise Exception("The solution is empty")
        if len(solution_list)<self.number_of_solutions_to_be_returned:
            raise Exception("The solution list contains less elements then requred")

        # random sampling without replacement
        return random.sample(solution_list, self.number_of_solutions_to_be_returned)


class RandomSolution(Selection[List[S], S]):
    def __init__(self):
        super(RandomSolution, self).__init__()

    def execute(self, solution_list: List[S]) -> S:
        if solution_list is None:
            raise Exception("The solution list is null")
        elif len(solution_list) == 0:
            raise Exception("The solution is empty")

        return random.choice(solution_list)


class RankingAndCrowdingDistanceSelection(Selection[List[S], List[S]]):
    def __init__(self, max_population_size: int):
        super(RankingAndCrowdingDistanceSelection, self).__init__()
        self.max_population_size = max_population_size

    def execute(self, solution_list: List[S]) -> List[S]:
        ranking = DominanceRanking()
        crowding_distance = CrowdingDistance()
        ranking.compute_ranking(solution_list)

        ranking_index = 0
        new_solution_list = []

        while len(new_solution_list) < self.max_population_size:
            if len(ranking.get_subfront(ranking_index)) < self.max_population_size - len(new_solution_list):
                new_solution_list = new_solution_list + ranking.get_subfront(ranking_index)
                ranking_index += 1
            else:
                subfront = ranking.get_subfront(ranking_index)
                crowding_distance.compute_density_estimator(subfront)
                sorted_subfront = sorted(subfront, key=lambda x: x.attributes["distance"], reverse=True)
                for i in range((self.max_population_size - len(new_solution_list))):
                    new_solution_list.append(sorted_subfront[i])

        return new_solution_list



from typing import TypeVar, List

from jmetal.util.comparator import dominance_comparator

S = TypeVar('S')


class Ranking(List[S]):
    def compute_ranking(self, solution_list: List[S]):
        pass

    def get_subfront(self, rank: int) -> List[S]:
        pass

    def get_number_of_subfronts(self) -> int:
        pass


class DominanceRanking(Ranking[List[S]]):
    def __init__(self):
        self.ranked_sublists = []

    def compute_ranking(self, solution_list: List[S]):
        # number of solutions dominating solution ith
        dominate_me = [0 for i in range(len(solution_list))]

        # list of solutions dominated by solution ith
        i_dominate = [[] for i in range(len(solution_list))]

        # front[i] contains the list of solutions belonging to front i
        front = [[] for i in range(len(solution_list) + 1)]

        for p in range(len(solution_list) - 1):
            for q in range(p + 1, len(solution_list)):
                dominance_test_result = dominance_comparator(solution_list[p], solution_list[q])
                if dominance_test_result is -1:
                    i_dominate[p].append(q)
                    dominate_me[q] += 1
                elif dominance_test_result is 1:
                    i_dominate[q].append(p)
                    dominate_me[p] += 1

        for i in range(len(solution_list)):
            if dominate_me[i] is 0:
                front[0].append(i)
                solution_list[i].attributes["ranking"] = 0

        i = 0
        while len(front[i]) != 0:
            i += 1
            for p in front[i - 1]:
                if p <= len(i_dominate):
                    for q in i_dominate[p]:
                        index = q
                        dominate_me[index] -= 1
                        if dominate_me[index] is 0:
                            front[i].append(index)
                            solution_list[index].attributes["ranking"] = i

        self.ranked_sublists = [[]] * i
        for j in range(i):
            Q = [0] * len(front[j])
            for k in range(len(front[j])):
                Q[k] = solution_list[front[j][k]]
            self.ranked_sublists[j] = Q

        return self.ranked_sublists

    def get_subfront(self, rank: int):
        if rank >= len(self.ranked_sublists):
            raise Exception("Invalid rank: " + str(rank) + ". Max rank = " + str(len(self.ranked_sublists) -1))
        return self.ranked_sublists[rank]

    def get_number_of_subfronts(self):
        return len(self.ranked_sublists)



import logging
from typing import List

from jmetal.algorithm.multiobjective.nsgaii import NSGAII
from jmetal.core.solution import FloatSolution
from jmetal.operator.crossover import SBX
from jmetal.operator.mutation import Polynomial
from jmetal.operator.selection import BinaryTournament
from jmetal.component.observer import AlgorithmObserver, WriteFrontToFileObserver
from jmetal.problem.multiobjective.unconstrained import Kursawe
from jmetal.util.solution_list_output import SolutionListOutput
from jmetal.util.time import get_time_of_execution

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


@get_time_of_execution
def main() -> None:
    problem = Kursawe()
    algorithm = NSGAII[FloatSolution, List[FloatSolution]](
        problem,
        population_size=100,
        max_evaluations=25000,
        mutation=Polynomial(1.0/problem.number_of_variables, distribution_index=20),
        crossover=SBX(1.0, distribution_index=20),
        selection=BinaryTournament())

    algorithm.run()
    result = algorithm.get_result()

    SolutionListOutput[FloatSolution].plot_scatter_to_file("FUN."+problem.get_name(), result)
    SolutionListOutput[FloatSolution].plot_scatter_to_screen(result)

    logger.info("Algorithm (continuous problem): " + algorithm.get_name())
    logger.info("Problem: " + problem.get_name())

if __name__ == '__main__':
    main()



from jmetal.algorithm.singleobjective.evolutionaryalgorithm import NonElitistEvolutionStrategy
from jmetal.core.solution import FloatSolution
from jmetal.operator.mutation import Polynomial
from jmetal.problem.singleobjective.unconstrained import Sphere


def main() -> None:
    variables = 10
    problem = Sphere(variables)
    algorithm = NonElitistEvolutionStrategy[FloatSolution, FloatSolution]\
        (problem, mu=10, lambdA=10, max_evaluations= 50000, mutation=Polynomial(1.0/variables))

    algorithm.run()
    result = algorithm.get_result()
    print("Algorithm: " + algorithm.get_name())
    print("Problem: " + problem.get_name())
    print("Solution: " + str(result.variables))
    print("Fitness:  " + str(result.objectives[0]))
    print("Computing time: " + str(algorithm.total_computing_time))


if __name__ == '__main__':
    main()



import unittest

from hamcrest import *

from jmetal.core.solution import Solution
from jmetal.operator.selection import BinaryTournament, BestSolution, RandomSolution, NaryRandomSolution, \
    RankingAndCrowdingDistanceSelection

__author__ = "Antonio J. Nebro"


class BinaryTournamentTestCases(unittest.TestCase):
    def setUp(self):
        self.selection = BinaryTournament[Solution]()

    def test_should_constructor_create_a_non_null_object(self):
        self.assertIsNotNone(self.selection)

    def test_should_execute_raise_an_exception_if_the_list_of_solutions_is_none(self):
        solution_list = None
        with self.assertRaises(Exception):
            self.selection.execute(solution_list)

    def test_should_execute_raise_an_exception_if_the_list_of_solutions_is_empty(self):
        solution_list = []
        with self.assertRaises(Exception):
            self.selection.execute(solution_list)

    def test_should_execute_return_the_solution_in_a_list_with_one_solution(self):
        solution = Solution(3,2)
        solution_list = [solution]

        self.assertEqual(solution, self.selection.execute(solution_list))

    def test_should_execute_work_if_the_solution_list_contains_two_non_dominated_solutions(self):
        solution1 = Solution(2,2)
        solution1.variables = [1.0, 2.0]
        solution2 = Solution(2,2)
        solution2.variables = [0.0, 3.0]

        solution_list = [solution1, solution2]

        assert_that(any_of(solution1 , solution2), self.selection.execute(solution_list))

    def test_should_execute_work_if_the_solution_list_contains_two_solutions_and_one_them_is_dominated(self):
        solution1 = Solution(2,2)
        solution1.variables = [1.0, 4.0]
        solution2 = Solution(2,2)
        solution2.variables = [0.0, 3.0]

        solution_list = [solution1, solution2]

        assert_that(solution2, self.selection.execute(solution_list))


class BestSolutionSelectionTestCases(unittest.TestCase):
    def setUp(self):
        self.selection = BestSolution[Solution]()

    def test_should_constructor_create_a_non_null_object(self):
        self.assertIsNotNone(self.selection)

    def test_should_execute_raise_an_exception_if_the_list_of_solutions_is_none(self):
        solution_list = None

        with self.assertRaises(Exception):
            self.selection.execute(solution_list)

    def test_should_execute_raise_an_exception_if_the_list_of_solutions_is_empty(self):
        solution_list = []
        with self.assertRaises(Exception):
            self.selection.execute(solution_list)

    def test_should_execute_return_the_solution_in_a_list_with_one_solution(self):
        solution = Solution(3,2)
        solution_list = [solution]

        self.assertEqual(solution, self.selection.execute(solution_list))

    def test_should_execute_work_if_the_solution_list_contains_two_non_dominated_solutions(self):
        solution1 = Solution(2,2)
        solution1.objectives = [1.0, 2.0]
        solution2 = Solution(2,2)
        solution2.objectives = [0.0, 3.0]

        solution_list = [solution1, solution2]

        self.assertTrue(self.selection.execute(solution_list) in solution_list)

    def test_should_execute_work_if_the_solution_list_contains_two_solutions_and_one_them_is_dominated(self):
        solution1 = Solution(2,2)
        solution1.objectives = [1.0, 4.0]
        solution2 = Solution(2,2)
        solution2.objectives = [0.0, 3.0]

        solution_list = [solution1, solution2]

        self.assertEqual(solution2, self.selection.execute(solution_list))

    def test_should_execute_work_if_the_solution_list_contains_five_solutions_and_one_them_is_dominated(self):
        solution1 = Solution(2,2)
        solution1.objectives = [1.0, 4.0]
        solution2 = Solution(2,2)
        solution2.objectives = [0.0, 3.0]
        solution3 = Solution(2,2)
        solution3.objectives = [0.0, 4.0]
        solution4 = Solution(2,2)
        solution4.objectives = [1.0, 3.0]
        solution5 = Solution(2,2)
        solution5.objectives = [0.2, 4.4]

        solution_list = [solution1, solution2, solution3, solution4, solution5]

        self.assertEqual(solution2, self.selection.execute(solution_list))


class RandomSolutionSelectionTestCases(unittest.TestCase):

    def setUp(self):
        self.selection = RandomSolution[Solution]()

    def test_should_constructor_create_a_non_null_object(self):
        self.assertIsNotNone(self.selection)

    def test_should_execute_raise_an_exception_if_the_list_of_solutions_is_none(self):
        solution_list = None
        with self.assertRaises(Exception):
            self.selection.execute(solution_list)

    def test_should_execute_raise_an_exception_if_the_list_of_solutions_is_empty(self):
        solution_list = []
        with self.assertRaises(Exception):
            self.selection.execute(solution_list)

    def test_should_execute_return_the_solution_in_a_list_with_one_solution(self):
        solution = Solution(3,2)
        solution_list = [solution]

        self.assertEqual(solution, self.selection.execute(solution_list))

    def test_should_execute_work_if_the_solution_list_contains_two_non_dominated_solutions(self):
        solution1 = Solution(2,2)
        solution1.objectives = [1.0, 2.0]
        solution2 = Solution(2,2)
        solution2.objectives = [0.0, 3.0]

        solution_list = [solution1, solution2]

        self.assertTrue(self.selection.execute(solution_list) in solution_list)

    def test_should_execute_work_if_the_solution_list_contains_two_solutions_and_one_them_is_dominated(self):
        solution1 = Solution(2,2)
        solution1.objectives = [1.0, 4.0]
        solution2 = Solution(2,2)
        solution2.objectives = [0.0, 3.0]

        solution_list = [solution1, solution2]

        self.assertTrue(self.selection.execute(solution_list) in solution_list)

    def test_should_execute_work_if_the_solution_list_contains_five_solutions_and_one_them_is_dominated(self):
        solution1 = Solution(2,2)
        solution1.objectives = [1.0, 4.0]
        solution2 = Solution(2,2)
        solution2.objectives = [0.0, 3.0]
        solution3 = Solution(2,2)
        solution3.objectives = [0.0, 4.0]
        solution4 = Solution(2,2)
        solution4.objectives = [1.0, 3.0]
        solution5 = Solution(2,2)
        solution5.objectives = [0.2, 4.4]

        solution_list = [solution1, solution2, solution3, solution4, solution5]
        self.assertTrue(self.selection.execute(solution_list) in solution_list)


class NaryRandomSolutionSelectionTestCases(unittest.TestCase):

    def setUp(self):
        pass

    def test_should_constructor_create_a_non_null_object(self):
        selection = NaryRandomSolution[Solution]()
        self.assertIsNotNone(selection)

    def test_should_constructor_create_a_non_null_object_and_check_number_of_elements(self):
        selection = NaryRandomSolution[Solution](3)
        self.assertEqual(selection.number_of_solutions_to_be_returned, 3)

    def test_should_execute_raise_an_exception_if_the_list_of_solutions_is_none(self):
        selection = NaryRandomSolution[Solution]()
        solution_list = None
        with self.assertRaises(Exception):
            selection.execute(solution_list)

    def test_should_execute_raise_an_exception_if_the_list_of_solutions_is_empty(self):
        selection = NaryRandomSolution[Solution]()
        solution_list = []
        with self.assertRaises(Exception):
           selection.execute(solution_list)

    def test_should_execute_raise_an_exception_if_the_list_of_solutions_is_smaller_than_required(self):
        selection = NaryRandomSolution[Solution](4)
        solution_list = [Solution(1,1), Solution(1,1)]
        with self.assertRaises(Exception):
           selection.execute(solution_list)

    def test_should_execute_return_the_solution_in_a_list_with_one_solution(self):
        selection = NaryRandomSolution[Solution](1)
        solution = Solution(3,2)
        solution_list = [solution]

        self.assertEqual([solution], selection.execute(solution_list))

    def test_should_execute_work_if_the_solution_list_contains_two_non_dominated_solutions(self):
        selection = NaryRandomSolution[Solution](2)
        solution1 = Solution(2,2)
        solution1.objectives = [1.0, 2.0]
        solution2 = Solution(2,2)
        solution2.objectives = [0.0, 3.0]

        solution_list = [solution1, solution2]

        selection_result = selection.execute(solution_list)
        self.assertTrue(selection_result[0] in solution_list)
        self.assertTrue(selection_result[1] in solution_list)

    def test_should_execute_work_if_the_solution_list_contains_five_solutions_and_one_them_is_dominated(self):
        selection = NaryRandomSolution[Solution](1)
        solution1 = Solution(2,2)
        solution1.objectives = [1.0, 4.0]
        solution2 = Solution(2,2)
        solution2.objectives = [0.0, 3.0]
        solution3 = Solution(2,2)
        solution3.objectives = [0.0, 4.0]
        solution4 = Solution(2,2)
        solution4.objectives = [1.0, 3.0]
        solution5 = Solution(2,2)
        solution5.objectives = [0.2, 4.4]

        solution_list = [solution1, solution2, solution3, solution4, solution5]

        self.assertTrue(selection.execute(solution_list)[0] in solution_list)


class DominanceRankingTestCases(unittest.TestCase):
    def setUp(self):
        self.ranking_and_crowding_selection = RankingAndCrowdingDistanceSelection(5)

    def test_should_len_of_nsgaii_execute_be_5(self):
        solution1 = Solution(2, 2)
        solution2 = Solution(2, 2)
        solution3 = Solution(2, 2)
        solution4 = Solution(2, 2)
        solution5 = Solution(2, 2)
        solution6 = Solution(2, 2)

        solution1.objectives[0] = 1.0
        solution1.objectives[1] = 0.0
        solution2.objectives[0] = 0.6
        solution2.objectives[1] = 0.6
        solution3.objectives[0] = 0.5
        solution3.objectives[1] = 0.5
        solution4.objectives[0] = 1.1
        solution4.objectives[1] = 0.0
        solution5.objectives[0] = 0.0
        solution5.objectives[1] = 1.0
        solution6.objectives[0] = 1.05
        solution6.objectives[1] = 0.1

        solution_list = [solution1, solution2, solution3, solution4, solution5, solution6]

        list_of_crowding_and_rankings = self.ranking_and_crowding_selection.execute(solution_list)

        self.assertEqual(len(list_of_crowding_and_rankings), 5)
        self.assertEqual(solution1, list_of_crowding_and_rankings[0])
        self.assertEqual(solution3, list_of_crowding_and_rankings[1])
        self.assertEqual(solution5, list_of_crowding_and_rankings[2])
        self.assertEqual(solution4, list_of_crowding_and_rankings[3])
        self.assertEqual(solution2, list_of_crowding_and_rankings[4])

if __name__ == '__main__':
    unittest.main()



import random

from jmetal.core.operator import Mutation
from jmetal.core.solution import BinarySolution, Solution, FloatSolution, IntegerSolution

""" Class implementing the binary BitFlip mutation operator """
__author__ = "Antonio J. Nebro"


class BitFlip(Mutation[BinarySolution]):
    def __init__(self, probability: float):
        super(BitFlip, self).__init__(probability=probability)

    def execute(self, solution: BinarySolution) -> BinarySolution:
        for i in range(solution.number_of_variables):
            for j in range(len(solution.variables[i])):
                rand = random.random()
                if rand <= self.probability:
                    solution.variables[i][j] = True if solution.variables[i][j] == False else False

        return solution


class Null(Mutation[Solution]):
    def __init__(self):
        super(Null, self).__init__(probability=0)

    def get_name(self):
        return "Null mutation"

    def execute(self, solution: Solution) -> Solution:
        return solution


class Polynomial(Mutation[FloatSolution]):
    def __init__(self, probability: float, distribution_index: float = 0.20):
        super(Polynomial, self).__init__(probability=probability)
        self.distribution_index = distribution_index

    def get_name(self):
        return "Polynomial mutation"

    def execute(self, solution: FloatSolution) -> FloatSolution:
        for i in range(solution.number_of_variables):
            rand = random.random()
            if rand <= self.probability:
                y = solution.variables[i]
                yl = solution.lower_bound[i]
                yu = solution.upper_bound[i]
                if yl == yu:
                    y = yl
                else:
                    delta1 = (y - yl) / (yu - yl)
                    delta2 = (yu - y) / (yu - yl)
                    rnd = random.random()
                    mut_pow = 1.0 / (self.distribution_index + 1.0)
                    if rnd <= 0.5:
                        xy = 1.0 - delta1
                        val = 2.0 * rnd + (1.0 - 2.0 * rnd) * (pow(xy, self.distribution_index + 1.0))
                        deltaq = pow(val, mut_pow) - 1.0
                    else:
                        xy = 1.0 - delta2
                        val = 2.0 * (1.0 - rnd) + 2.0 * (rnd - 0.5) * (pow(xy, self.distribution_index + 1.0));
                        deltaq = 1.0 - pow(val, mut_pow)

                    y += deltaq * (yu - yl)
                    if y < solution.lower_bound[i]:
                        y = solution.lower_bound[i]
                    if y > solution.upper_bound[i]:
                        y = solution.upper_bound[i]

                solution.variables[i] = y

        return solution


class IntegerPolynomial(Mutation[IntegerSolution]):
    def __init__(self, probability: float, distribution_index: float = 0.20):
        super(IntegerPolynomial, self).__init__(probability=probability)
        self.distribution_index = distribution_index

    def get_name(self):
        return "Polynomial mutation (Integer)"

    def execute(self, solution: IntegerSolution) -> IntegerSolution:
        for i in range(solution.number_of_variables):
            if random.random() <= self.probability:
                y = solution.variables[i]
                yl = solution.lower_bound[i]
                yu = solution.upper_bound[i]
                if yl == yu:
                    y = yl
                else:
                    delta1 = (y - yl) / (yu - yl)
                    delta2 = (yu - y) / (yu - yl)
                    mutPow = 1.0 / (self.distribution_index + 1.0)
                    rnd = random.random()
                    if rnd<=0.5:
                        xy = 1.0 - delta1
                        val = 2.0 * rnd + (1.0 - 2.0 * rnd) * (xy ** (self.distribution_index + 1.0))
                        deltaq = val**mutPow - 1.0
                    else:
                        xy = 1.0 - delta2
                        val = 2.0 * (1.0 - rnd) + 2.0 * (rnd - 0.5) * (xy**(self.distribution_index + 1.0))
                        deltaq = 1.0 - val**mutPow

                    y += deltaq * (yu - yl)
                    if y < solution.lower_bound[i]:
                        y = solution.lower_bound[i]
                    if y > solution.upper_bound[i]:
                        y = solution.upper_bound[i]

                solution.variables[i] = int(round(y))
        return solution


class SimpleRandom(Mutation[FloatSolution]):
    def __init__(self, probability: float):
        super(SimpleRandom, self).__init__(probability=probability)

    def get_name(self):
        return "Simple random mutation"

    def execute(self, solution: FloatSolution) -> FloatSolution:
        for i in range(solution.number_of_variables):
            rand = random.random()
            if rand <= self.probability:
                solution.variables[i] = solution.lower_bound[i] + (solution.upper_bound[i] - solution.lower_bound[i]) * random.random()
        return solution


class Uniform(Mutation[FloatSolution]):
    def __init__(self, probability: float, perturbation: float = 0.5):
        super(Uniform, self).__init__(probability=probability)
        self.perturbation = perturbation

    def get_name(self):
        return "Uniform mutation"

    def execute(self, solution: FloatSolution) -> FloatSolution:
        for i in range(solution.number_of_variables):
            rand = random.random()
            if rand <= self.probability:
                tmp = (random.random() - 0.5) * self.perturbation;
                tmp+= solution.variables[i]

                if tmp < solution.lower_bound[i]:
                    tmp = solution.lower_bound[i]
                elif tmp > solution.upper_bound[i]:
                    tmp = solution.upper_bound[i]

                solution.variables[i] = tmp

        return solution


from copy import deepcopy
from typing import TypeVar, List

from jmetal.core.algorithm import EvolutionaryAlgorithm
from jmetal.core.operator import Mutation, Crossover, Selection
from jmetal.core.problem import Problem
from jmetal.util.observable import Observable, DefaultObservable

""" Class representing elitist evolution strategy algorithms """

S = TypeVar('S')
R = TypeVar('R')


class ElitistEvolutionStrategy(EvolutionaryAlgorithm[S, R]):
    def __init__(self,
                 problem: Problem[S],
                 mu: int,
                 lambdA: int,
                 max_evaluations: int,
                 mutation: Mutation[S]):
        super(ElitistEvolutionStrategy, self).__init__()
        self.problem = problem
        self.mu = mu
        self.lambdA = lambdA
        self.max_evaluations = max_evaluations
        self.mutation = mutation

    def init_progress(self):
        self.evaluations = self.mu

    def update_progress(self):
        self.evaluations += self.lambdA

    def is_stopping_condition_reached(self) -> bool:
        return self.evaluations >= self.max_evaluations

    def create_initial_population(self) -> List[S]:
        population = []
        for i in range(self.mu):
            population.append(self.problem.create_solution())
        return population

    def evaluate_population(self, population: List[S]):
        for solution in population:
            self.problem.evaluate(solution)
        return population

    def selection(self, population: List[S]):
        return population

    def reproduction(self, population: List[S]):
        offspring_population = []
        for solution in population:
            for j in range((int)(self.lambdA / self.mu)):
                new_solution = deepcopy(solution)
                offspring_population.append(self.mutation.execute(new_solution))

        return offspring_population

    def replacement(self, population: List[S], offspring_population: List[S]) \
            -> List[S]:
        for solution in offspring_population:
            self.population.append(solution)

        population.sort(key=lambda s: s.objectives[0])

        new_population = []
        for i in range(self.mu):
            new_population.append(population[i])

        return new_population

    def get_result(self) -> R:
        return self.population[0]

    def get_name(self):
        return "(" + str(self.mu) + "+" + str(self.lambdA) + ")ES"


class NonElitistEvolutionStrategy(ElitistEvolutionStrategy[S, R]):
    def __init__(self,
                 problem: Problem[S],
                 mu: int, lambdA: int,
                 max_evaluations: int,
                 mutation: Mutation[S]):
        super(NonElitistEvolutionStrategy, self).__init__(problem, mu, lambdA,
                                                          max_evaluations, mutation)

    def replacement(self, population: List[S], offspring_population: List[S]) \
            -> List[S]:
        offspring_population.sort(key=lambda s: s.objectives[0])

        new_population = []
        for i in range(self.mu):
            new_population.append(offspring_population[i])

        return new_population

    def get_name(self) -> str:
        return "(" + str(self.mu) + "," + str(self.lambdA) + ")ES"


class GenerationalGeneticAlgorithm(EvolutionaryAlgorithm[S, R]):
    def __init__(self,
                 problem: Problem[S],
                 population_size: int,
                 max_evaluations: int,
                 mutation: Mutation[S],
                 crossover: Crossover[S, S],
                 selection: Selection[List[S], S],
                 observable: Observable = DefaultObservable()):
        super(GenerationalGeneticAlgorithm, self).__init__()
        self.problem = problem
        self.population_size = population_size
        self.max_evaluations = max_evaluations
        self.mutation_operator = mutation
        self.crossover_operator = crossover
        self.selection_operator = selection
        self.evaluations = 0
        self.observable = observable

    def init_progress(self):
        self.evaluations = self.population_size

    def update_progress(self):
        self.evaluations += self.population_size
        observable_data = {'evaluations': self.evaluations,
                           'population': self.population}
        self.observable.notify_all(**observable_data)

    def is_stopping_condition_reached(self) -> bool:
        return self.evaluations >= self.max_evaluations

    def create_initial_population(self) -> List[S]:
        population = []

        for i in range(self.population_size):
            population.append(self.problem.create_solution())

        return population

    def evaluate_population(self, population: List[S]):
        for solution in population:
            self.problem.evaluate(solution)
        return population

    def selection(self, population: List[S]):
        mating_population = []

        for i in range(self.population_size):
            solution = self.selection_operator.execute(self.population)
            mating_population.append(solution)

        return mating_population

    def reproduction(self, population: List[S]) -> List[S]:
        number_of_parents_to_combine = self.crossover_operator.get_number_of_parents()
        self.__check_number_of_parents(population, number_of_parents_to_combine)

        offspring_population = []
        for i in range(0, self.population_size, number_of_parents_to_combine):
            parents = []
            for j in range(number_of_parents_to_combine):
                parents.append(population[i + j])

            offspring = self.crossover_operator.execute(parents)

            for solution in offspring:
                self.mutation_operator.execute(solution)
                offspring_population.append(solution)

        return offspring_population

    def replacement(self, population: List[S], offspring_population: List[S]) \
            -> List[S]:
        population.sort(key=lambda s: s.objectives[0])

        offspring_population.append(population[0])
        offspring_population.append(population[1])

        offspring_population.sort(key=lambda s: s.objectives[0])

        offspring_population.pop()
        offspring_population.pop()

        return offspring_population

    def get_result(self) -> R:
        """ :return: The best individual of the population.
        """
        return self.population[0]

    def get_name(self) -> str:
        return "Generational Genetic Algorithm"

    def __check_number_of_parents(self, population: List[S], number_of_parents_for_crossover: int):
        if self.population_size % number_of_parents_for_crossover != 0:
            raise Exception("Wrong number of parents")




from abc import ABCMeta, abstractmethod


class Observable(object):
    def register(self, observer):
        pass

    def deregister(self, observer):
        pass

    def deregister_all(self):
        pass

    def notify_all(self, *args, **kwargs):
        pass


class Observer(object):
    __metaclass__ = ABCMeta

    @abstractmethod
    def update(self, *args, **kwargs):
        """
        Update method
        :param args:
        :param kwargs:
        :return:
        """
        pass


class DefaultObservable(Observable):
    def __init__(self):
        self.observers = []

    def register(self, observer):
        if observer not in self.observers:
            self.observers.append(observer)

    def deregister(self, observer):
        if observer in self.observers:
            self.observers.remove(observer)

    def deregister_all(self):
        if self.observers:
            del self.observers[:]

    def notify_all(self, *args, **kwargs):
        for observer in self.observers:
            observer.update(*args, **kwargs)



from setuptools import setup, find_packages

setup(name='jmetalpy',
      version='0.1',
      description='JMetalPy. Python version of the jMetal framework',
      author='Antonio J. Nebro',
      author_email='ajnebro@uma.es',
      maintainer='Antonio Nebro, Carlos Canicio',
      maintainer_email='ajnebro@uma.es, canicio@lcc.uma.es',
      license='MIT',
      url='https://github.com/jMetal/jMetalPy',
      classifiers=[
          'Development Status :: 3 - Alpha',

          'Intended Audience :: Science/Research',

          'License :: OSI Approved :: MIT License',

          'Topic :: Scientific/Engineering :: Artificial Intelligence',

          'Programming Language :: Python :: 3.6'],

      packages=find_packages(exclude=['test_']),
      )



import unittest

from jmetal.core.solution import FloatSolution
from jmetal.util.comparator import dominance_comparator


class DominanceComparatorTestCases(unittest.TestCase):

    def setUp(self):
        pass

    def test_should_dominance_comparator_raise_an_exception_if_the_first_solution_is_null(self):
        solution = None
        solution2 = FloatSolution(3, 2, 0, [], [])
        with self.assertRaises(Exception):
            dominance_comparator(solution, solution2)

    def test_should_dominance_comparator_raise_an_exception_if_the_second_solution_is_null(self):
        solution = FloatSolution(3, 2, 0, [], [])
        solution2 = None
        with self.assertRaises(Exception):
            dominance_comparator(solution, solution2)

    def test_should_dominance_comparator_raise_an_exception_if_the_solutions_have_not_the_same_number_of_objectives(self):
        solution = FloatSolution(3, 2, 0, [], [])
        solution2 = FloatSolution(3, 5, 0, [], [])
        with self.assertRaises(Exception):
            dominance_comparator(solution, solution2)

    def test_should_dominance_comparator_return_zero_if_the_two_solutions_have_one_objective_with_the_same_value(self):
        solution = FloatSolution(3, 1, 0, [], [])
        solution2 = FloatSolution(3, 1, 0, [], [])

        solution.objectives = [1.0]
        solution2.objectives = [1.0]

        self.assertEqual(0, dominance_comparator(solution, solution2))

    def test_should_dominance_comparator_return_one_if_the_two_solutions_have_one_objective_and_the_second_one_is_lower(self):
        solution = FloatSolution(3, 1, 0, [], [])
        solution2 = FloatSolution(3, 1, 0, [], [])

        solution.objectives = [2.0]
        solution2.objectives = [1.0]

        self.assertEqual(1, dominance_comparator(solution, solution2))

    def test_should_dominance_comparator_return_minus_one_if_the_two_solutions_have_one_objective_and_the_first_one_is_lower(self):
        solution = FloatSolution(3, 1, 0, [], [])
        solution2 = FloatSolution(3, 1, 0, [], [])

        solution.objectives = [1.0]
        solution2.objectives = [2.0]

        self.assertEqual(-1, dominance_comparator(solution, solution2))

    def test_should_dominance_comparator_work_properly_case_a(self):
        '''
        Case A: solution1 has objectives [-1.0, 5.0, 9.0] and solution2 has [2.0, 6.0, 15.0]
        '''
        solution = FloatSolution(3, 3, 0, [], [])
        solution2 = FloatSolution(3, 3, 0, [], [])

        solution.objectives = [-1.0, 5.0, 9.0]
        solution2.objectives = [2.0, 6.0, 15.0]

        self.assertEqual(-1, dominance_comparator(solution, solution2))

    def test_should_dominance_comparator_work_properly_case_b(self):
        '''
        Case b: solution1 has objectives [-1.0, 5.0, 9.0] and solution2 has [-1.0, 5.0, 10.0]
        '''
        solution = FloatSolution(3, 3, 0, [], [])
        solution2 = FloatSolution(3, 3, 0, [], [])

        solution.objectives = [-1.0, 5.0, 9.0]
        solution2.objectives = [-1.0, 5.0, 10.0]

        self.assertEqual(-1, dominance_comparator(solution, solution2))

    def test_should_dominance_comparator_work_properly_case_c(self):
        '''
        Case c: solution1 has objectives [-1.0, 5.0, 9.0] and solution2 has [-2.0, 5.0, 9.0]
        '''
        solution = FloatSolution(3, 3, 0, [], [])
        solution2 = FloatSolution(3, 3, 0, [], [])

        solution.objectives = [-1.0, 5.0, 9.0]
        solution2.objectives = [-2.0, 5.0, 9.0]

        self.assertEqual(1, dominance_comparator(solution, solution2))

    def test_should_dominance_comparator_work_properly_case_d(self):
        '''
        Case d: solution1 has objectives [-1.0, 5.0, 9.0] and solution2 has [-1.0, 5.0, 8.0]
        '''
        solution = FloatSolution(3, 3, 0, [], [])
        solution2 = FloatSolution(3, 3, 0 , [], [])

        solution.objectives = [-1.0, 5.0, 9.0]
        solution2.objectives = [-1.0, 5.0, 8.0]

        self.assertEqual(1, dominance_comparator(solution, solution2))

    def test_should_dominance_comparator_work_properly_case_3(self):
        '''
        Case d: solution1 has objectives [-1.0, 5.0, 9.0] and solution2 has [-2.0, 5.0, 10.0]
        '''
        solution = FloatSolution(3, 3, 0, [], [])
        solution2 = FloatSolution(3, 3, 0, [], [])

        solution.objectives = [-1.0, 5.0, 9.0]
        solution2.objectives = [-2.0, 5.0, 10.0]

        self.assertEqual(0, dominance_comparator(solution, solution2))

if __name__ == '__main__':
    unittest.main()




import logging
from typing import List

from jmetal.algorithm.multiobjective.nsgaii import NSGAII
from jmetal.component.consumer import BasicAlgorithmConsumer
from jmetal.core.solution import FloatSolution
from jmetal.operator.crossover import SBX
from jmetal.operator.mutation import Polynomial
from jmetal.operator.selection import BinaryTournament
from jmetal.problem.multiobjective.zdt import ZDT1
from jmetal.util.solution_list_output import SolutionListOutput

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


def main() -> None:
    problem = ZDT1()
    algorithm = NSGAII[FloatSolution, List[FloatSolution]](
        problem,
        population_size = 100,
        max_evaluations = 25000,
        mutation = Polynomial(1.0/problem.number_of_variables, distribution_index=20),
        crossover = SBX(1.0, distribution_index=20),
        selection = BinaryTournament())

    observer = BasicAlgorithmConsumer(1000)

    algorithm.observable.register(observer=observer)

    algorithm.run()

    result = algorithm.get_result()

    SolutionListOutput[FloatSolution].print_function_values_to_file("FUN."+problem.get_name(), result)

    logger.info("Algorithm (continuous problem): " + algorithm.get_name())
    logger.info("Problem: " + problem.get_name())
    logger.info("Computing time: " + str(algorithm.total_computing_time))

if __name__ == '__main__':
    main()



import unittest

from jmetal.core.solution import BinarySolution, FloatSolution, Solution

__author__ = "Antonio J. Nebro"

class BinarySolutionTestCase(unittest.TestCase):

    def setUp(self) -> None:
        pass

    def test_should_default_constructor_create_a_valid_solution(self) -> None:
        solution = BinarySolution(2, 3, 1)
        self.assertEqual(2, solution.number_of_variables)
        self.assertEqual(3, solution.number_of_objectives)
        self.assertEqual(1, solution.number_of_constraints)

    def test_should_constructor_create_a_valid_solution(self) -> None:
        solution = BinarySolution(number_of_variables=2, number_of_objectives=3)
        solution.variables[0] = [True, False]
        solution.variables[1] = [False]
        self.assertEqual(2, solution.number_of_variables)
        self.assertEqual(3, solution.number_of_objectives)
        self.assertEqual(2, len(solution.variables))
        self.assertEqual(3, len(solution.objectives))
        self.assertEqual([True, False], solution.variables[0])
        self.assertEqual([False], solution.variables[1])

    def test_should_get_total_number_of_bits_return_zero_if_the_object_variables_are_not_initialized(self) -> None:
        solution = BinarySolution(number_of_variables=2, number_of_objectives=3)
        self.assertEqual(0, solution.get_total_number_of_bits())

    def test_should_get_total_number_of_bits_return_the_right_value(self) -> None:
        solution = BinarySolution(number_of_variables=2, number_of_objectives=3)
        solution.variables[0] = [True, False]
        solution.variables[1] = [False, True, False]
        self.assertEqual(5, solution.get_total_number_of_bits())


class FloatSolutionTestCase(unittest.TestCase):

    def setUp(self) -> None:
        pass

    def test_should_constructor_create_a_non_null_object(self) -> None:
        solution = FloatSolution(3, 2, 0, [], [])
        self.assertIsNotNone(solution)

    def test_should_default_constructor_create_a_valid_solution(self) -> None:
        solution = FloatSolution(2, 3, 2, [0.0, 0.5], [1.0, 2.0])
        self.assertEqual(2, solution.number_of_variables)
        self.assertEqual(3, solution.number_of_objectives)
        self.assertEqual(2, solution.number_of_constraints)
        self.assertEqual(2, len(solution.variables))
        self.assertEqual(3, len(solution.objectives))
        self.assertEqual([0.0, 0.5], solution.lower_bound)
        self.assertEqual([1.0, 2.0], solution.upper_bound)


class SolutionTestCase(unittest.TestCase):

    def setUp(self) -> None:
        pass

    def test_should_constructor_create_a_non_null_object(self) -> None:
        solution = Solution[int](3, 2)
        self.assertIsNotNone(solution)

    def test_should_constructor_create_a_valid_solution_of_ints(self) -> None:
        solution = Solution[int](3, 2)
        self.assertEqual(3, solution.number_of_variables)
        self.assertEqual(2, solution.number_of_objectives)
        self.assertEqual(0, solution.number_of_constraints)

    def test_should_constructor_create_a_valid_solution_of_floats(self) -> None:
        solution = Solution[float](3, 2, 5)
        self.assertEqual(3, solution.number_of_variables)
        self.assertEqual(2, solution.number_of_objectives)
        self.assertEqual(5, solution.number_of_constraints)

    def test_should_constructor_create_a_non_null_objective_list(self) -> None:
        solution = Solution[float](3, 2)
        self.assertIsNotNone(solution.objectives)


if __name__ == '__main__':
    unittest.main()



import unittest
from unittest import mock

from jmetal.core.solution import BinarySolution
from jmetal.operator.crossover import Null, SinglePoint


class NullCrossoverTestCases(unittest.TestCase):

    def setUp(self):
        pass

    def test_should_constructor_create_a_non_null_object(self):
        solution = Null()
        self.assertIsNotNone(solution)

    def test_should_constructor_create_a_valid_operator(self):
        operator = Null()
        self.assertEqual(0, operator.probability)

    def test_should_the_solution_remain_unchanged(self):
        operator = Null()
        solution1 = BinarySolution(number_of_variables=1, number_of_objectives=1)
        solution1.variables[0] = [True, False, False, True, True, False]
        solution2 = BinarySolution(number_of_variables=1, number_of_objectives=1)
        solution2.variables[0] = [False, True, False, False, True, False]

        offspring = operator.execute([solution1, solution2])
        self.assertEqual([True, False, False, True, True, False], offspring[0].variables[0])
        self.assertEqual([False, True, False, False, True, False], offspring[1].variables[0])


class SinglePointTestCases(unittest.TestCase):
    def setUp(self):
        pass

    def test_should_constructor_create_a_non_null_object(self):
        solution = SinglePoint(1.0)
        self.assertIsNotNone(solution)

    def test_should_constructor_create_a_valid_operator(self):
        operator = SinglePoint(0.5)
        self.assertEqual(0.5, operator.probability)

    def test_should_constructor_raise_an_exception_if_the_probability_is_greater_than_one(self):
        with self.assertRaises(Exception):
            SinglePoint(2)

    def test_should_constructor_raise_an_exception_if_the_probability_is_lower_than_zero(self):
        with self.assertRaises(Exception):
            SinglePoint(-12)

    def test_should_the_solution_remain_unchanged_if_the_probability_is_zero(self):
        operator = SinglePoint(0.0)
        solution1 = BinarySolution(number_of_variables=1, number_of_objectives=1)
        solution1.variables[0] = [True, False, False, True, True, False]
        solution2 = BinarySolution(number_of_variables=1, number_of_objectives=1)
        solution2.variables[0] = [False, True, False, False, True, False]

        offspring = operator.execute([solution1, solution2])
        self.assertEqual([True, False, False, True, True, False], offspring[0].variables[0])
        self.assertEqual([False, True, False, False, True, False], offspring[1].variables[0])

    @mock.patch('random.randrange')
    def test_should_the_operator_work_if_the_first_bit_is_selected(self, random_call):
        operator = SinglePoint(1.0)
        solution1 = BinarySolution(number_of_variables=1, number_of_objectives=1)
        solution1.variables[0] = [True, False, False, True, True, False]
        solution2 = BinarySolution(number_of_variables=1, number_of_objectives=1)
        solution2.variables[0] = [False, True, False, False, True, False]

        random_call.return_value = 0
        offspring = operator.execute([solution1, solution2])
        self.assertEqual([False, True, False, False, True, False], offspring[0].variables[0])
        self.assertEqual([True, False, False, True, True, False], offspring[1].variables[0])

    @mock.patch('random.randrange')
    def test_should_the_operator_work_if_the_last_bit_is_selected(self, random_call):
        operator = SinglePoint(1.0)
        solution1 = BinarySolution(number_of_variables=1, number_of_objectives=1)
        solution1.variables[0] = [True, False, False, True, True, False]
        solution2 = BinarySolution(number_of_variables=1, number_of_objectives=1)
        solution2.variables[0] = [False, True, False, False, True, True]

        random_call.return_value = 5
        offspring = operator.execute([solution1, solution2])
        self.assertEqual([True, False, False, True, True, True], offspring[0].variables[0])
        self.assertEqual([False, True, False, False, True, False], offspring[1].variables[0])

    @mock.patch('random.randrange')
    def test_should_the_operator_work_if_the_third_bit_is_selected(self, random_call):
        operator = SinglePoint(1.0)
        solution1 = BinarySolution(number_of_variables=1, number_of_objectives=1)
        solution1.variables[0] = [True, False, False, True, True, False]
        solution2 = BinarySolution(number_of_variables=1, number_of_objectives=1)
        solution2.variables[0] = [False, True, False, False, True, True]

        random_call.return_value = 3
        offspring = operator.execute([solution1, solution2])
        self.assertEqual([True, False, False, False, True, True], offspring[0].variables[0])
        self.assertEqual([False, True, False, True, True, False], offspring[1].variables[0])

    @mock.patch('random.randrange')
    def test_should_the_operator_work_with_a_solution_with_three_binary_variables(self, random_call):
        operator = SinglePoint(1.0)
        solution1 = BinarySolution(number_of_variables=3, number_of_objectives=1)
        solution1.variables[0] = [True, False, False, True, True, False]
        solution1.variables[1] = [True, False, False, True, False, False]
        solution1.variables[2] = [True, False, True, True, True, True]
        solution2 = BinarySolution(number_of_variables=3, number_of_objectives=1)
        solution2.variables[0] = [False, True, False, False, True, True]
        solution2.variables[1] = [True, True, False, False, True, False]
        solution2.variables[2] = [True, True, True, False, False, True]

        random_call.return_value = 8
        offspring = operator.execute([solution1, solution2])
        self.assertEqual([True, False, False, True, True, False], offspring[0].variables[0])
        self.assertEqual([True, False, False, False, True, False], offspring[0].variables[1])
        self.assertEqual([True, True, True, False, False, True], offspring[0].variables[2])
        self.assertEqual([False, True, False, False, True, True], offspring[1].variables[0])
        self.assertEqual([True, True, False, True, False, False], offspring[1].variables[1])
        self.assertEqual([True, False, True, True, True, True], offspring[1].variables[2])

if __name__ == '__main__':
    unittest.main()



import logging
from typing import TypeVar, List, Tuple

import matplotlib.pyplot as plt

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

S = TypeVar('S')

"""
.. module:: graphics
   :platform: Unix, Windows
   :synopsis: Class for plotting solutions.

.. moduleauthor:: Antonio Benítez <antonio.b@uma.es>
"""


class ScatterPlot():
    """ Scatter plot.
    """
    def __init__(self, plot_title: str, animation_speed: float = 1*10e-10):
        """ Creates a new :class:`ScatterPlot` instance.
        Args:
           plot_title (str): Title of the scatter diagram.
           animation_speed (float): Delay (for live plot only). Allow time for the gui event loops to trigger
           and update the display.
        """
        self.plot_title = plot_title
        self.fig = plt.figure()
        self.axis = self.fig.add_subplot(111)
        self.sc = None

        # Real-time plotting options
        self.animation_speed = animation_speed

    def __init_plot(self, is_auto_scalable: bool = True) -> None:
        if is_auto_scalable:
            self.axis.set_autoscale_on(True)
            self.axis.autoscale_view(True, True, True)

        logger.info("Generating plot...")

        # Style options
        self.axis.grid(color='#f0f0f5', linestyle='-', linewidth=2, alpha=0.5)
        self.fig.suptitle(self.plot_title, fontsize=14, fontweight='bold')

    def __get_data_points(self, solution_list: List[S]) -> Tuple[list, list]:
        """ Get coords (x,y) from a solution_list. """
        points = list(sol.objectives for sol in solution_list)
        x_values, y_values = [x[0] for x in points], [y[1] for y in points]

        return x_values, y_values

    def __search_solution(self, solution_list: List[S], x_val: float, y_val: float) -> None:
        """ Return a solution object associated with some values of (x,y). """
        for solution in solution_list:
            if solution.objectives[0] == x_val and solution.objectives[1] == y_val:
                logger.info('Solution associated to ({0}, {1}): {2}'
                            .format(x_val, y_val, solution))

    def __pick_handler(self, event, solution_list: List[S]):
        line, ind = event.artist, event.ind[0]
        x, y = line.get_xdata(), line.get_ydata()

        logger.info('Selected data point ({0}): ({1}, {2})'.format(ind, x[ind], y[ind]))
        self.__search_solution(solution_list, x[ind], y[ind])

    def simple_plot(self, solution_list: List[S], file_name: str = "output",
                    fmt: str = 'ps', dpi: int = 200, save: bool = True) -> None:

        self.__init_plot()
        x_values, y_values = self.__get_data_points(solution_list)

        self.sc, = self.axis.plot(x_values, y_values, 'go', markersize=5, picker=10)

        if save:
            self.fig.savefig(file_name + '.' + fmt, format=fmt, dpi=dpi)
            logger.info("(Supported formats: eps, jpeg, jpg, pdf, pgf, png, ps, raw, rgba, svg, svgz, tif, tiff)")
            logger.info("Output file (function plot): " + file_name + '.' + fmt)

    def interactive_plot(self, solution_list: List[S]) -> None:
        self.__init_plot()
        x_values, y_values = self.__get_data_points(solution_list)

        self.sc, = self.axis.plot(x_values, y_values, 'go', markersize=5, picker=10)
        self.fig.canvas.mpl_connect('pick_event', lambda event: self.__pick_handler(event, solution_list))

        plt.show()

    def update(self, solution_list: List[S]) -> None:
        x_values, y_values = self.__get_data_points(solution_list)

        self.sc.set_data(x_values, y_values)
        event_handler = self.fig.canvas.mpl_connect('pick_event', lambda event: self.__pick_handler(event, solution_list))

        self.axis.relim()
        self.axis.autoscale_view(True, True, True)

        self.fig.canvas.draw()
        plt.pause(self.animation_speed)

        # Disconnect the pick event for the loop
        self.fig.canvas.mpl_disconnect(event_handler)



from jmetal.core.solution import Solution


def dominance_comparator(solution1: Solution, solution2: Solution) -> int:
    if solution1 is None:
        raise Exception("The solution1 is None")
    elif solution2 is None:
        raise Exception("The solution2 is None")
    elif len(solution1.objectives) != len(solution2.objectives):
        raise Exception("The solutions have different number of objectives")

    best_is_one = 0
    best_is_two = 0

    for i in range(solution1.number_of_objectives):
        value1 = solution1.objectives[i]
        value2 = solution2.objectives[i]
        if value1 != value2:
            if value1 < value2:
                best_is_one = 1
            if value2 < value1:
                best_is_two = 1

    if best_is_one > best_is_two:
        result = -1
    elif best_is_two > best_is_one:
        result = 1
    else:
        result = 0

    return result


def equal_solutions_comparator(solution1: Solution, solution2: Solution) -> int:
    if solution1 is None:
        return 1
    elif solution2 is None:
        return -1

    dominate1 = 0
    dominate2 = 0

    for i in range(len(solution1.objectives)):
        value1 = solution1.objectives[i]
        value2 = solution2.objectives[i]

        if value1<value2:
            flag = -1
        elif value1 > value2:
            flag = 1
        else:
            flag = 0

        if flag == -1:
            dominate1 = 1
        if flag == 1:
            dominate2 = 1

    if dominate1 == 0 and dominate2 == 0:
        return 0
    elif dominate1 == 1:
        return -1
    elif dominate2 == 1:
        return 1